@@ -403,7 +403,7 @@ bool cephx_verify_authorizer(CephContext *cct, KeyStore *keys,
     ::decode(global_id, indata);
     ::decode(service_id, indata);
     ::decode(ticket, indata);
-  } catch (buffer::end_of_buffer e) {
+  } catch (buffer::end_of_buffer &e) {
     // Unable to decode!
     return false;
   }
@@ -423,6 +423,8 @@ bool cephx_verify_authorizer(CephContext *cct, KeyStore *keys,
     if (!keys->get_service_secret(service_id, ticket.secret_id, service_secret)) {
       ldout(cct, 0) << "verify_authorizer could not get service secret for service "
 	      << ceph_entity_type_name(service_id) << " secret_id=" << ticket.secret_id << dendl;
+      if (cct->_conf->auth_debug && ticket.secret_id == 0)
+	assert(0 == "got secret_id=0");
       return false;
     }
   }
