@@ -17,7 +17,6 @@
 #include <fcntl.h>
 #include <uuid/uuid.h>
 
-#include <sys/stat.h>
 #include <iostream>
 #include <string>
 using namespace std;
@@ -265,6 +264,14 @@ int main(int argc, const char **argv)
     exit(0);
   }
   
+  if (get_journal_fsid) {
+    uuid_d fsid;
+    int r = OSD::peek_journal_fsid(g_conf->osd_journal, fsid);
+    if (r == 0)
+      cout << fsid << std::endl;
+    exit(r);
+  }
+
   string magic;
   uuid_d cluster_fsid, osd_fsid;
   int w;
@@ -297,13 +304,6 @@ int main(int argc, const char **argv)
     cout << osd_fsid << std::endl;
     exit(0);
   }
-  if (get_journal_fsid) {
-    uuid_d fsid;
-    int r = OSD::peek_journal_fsid(g_conf->osd_journal, fsid);
-    if (r == 0)
-      cout << fsid << std::endl;
-    exit(r);
-  }
 
   pick_addresses(g_ceph_context);
 
@@ -343,7 +343,8 @@ int main(int argc, const char **argv)
   uint64_t supported =
     CEPH_FEATURE_UID | 
     CEPH_FEATURE_NOSRCADDR |
-    CEPH_FEATURE_PGID64;
+    CEPH_FEATURE_PGID64 |
+    CEPH_FEATURE_MSG_AUTH;
 
   client_messenger->set_default_policy(Messenger::Policy::stateless_server(supported, 0));
   client_messenger->set_policy_throttler(entity_name_t::TYPE_CLIENT, &client_throttler);  // default, actually
@@ -457,6 +458,7 @@ int main(int argc, const char **argv)
   delete messenger_hbclient;
   delete messenger_hbserver;
   delete cluster_messenger;
+  g_ceph_context->put();
 
   // cd on exit, so that gmon.out (if any) goes into a separate directory for each node.
   char s[20];
