@@ -217,7 +217,7 @@ class Client : public Dispatcher {
   Context *tick_event;
   utime_t last_cap_renew;
   void renew_caps();
-  void renew_caps(int s);
+  void renew_caps(MetaSession *session);
   void flush_cap_releases();
 public:
   void tick();
@@ -229,14 +229,18 @@ public:
 
   // mds sessions
   map<int, MetaSession*> mds_sessions;  // mds -> push seq
-  map<int, list<Cond*> > waiting_for_session;
   list<Cond*> waiting_for_mdsmap;
 
-  void got_mds_push(int mds);
-  void _closed_mds_session(int mds, MetaSession *s);
+  bool have_open_session(int mds);
+  void got_mds_push(MetaSession *s);
+  MetaSession *_get_mds_session(int mds, Connection *con);  ///< return session for mds *and* con; null otherwise
+  MetaSession *_get_or_open_mds_session(int mds);
+  MetaSession *_open_mds_session(int mds);
+  void _close_mds_session(MetaSession *s);
+  void _closed_mds_session(MetaSession *s);
   void handle_client_session(MClientSession *m);
-  void send_reconnect(int mds);
-  void resend_unsafe_requests(int mds);
+  void send_reconnect(MetaSession *s);
+  void resend_unsafe_requests(MetaSession *s);
 
   // mds requests
   tid_t last_tid, last_flush_seq;
@@ -248,8 +252,10 @@ public:
 
   int make_request(MetaRequest *req, int uid, int gid,
 		   //MClientRequest *req, int uid, int gid,
-		   Inode **ptarget = 0,
+		   Inode **ptarget = 0, bool *pcreated = 0,
 		   int use_mds=-1, bufferlist *pdirbl=0);
+  int verify_reply_trace(int r, MetaRequest *request, MClientReply *reply,
+			 Inode **ptarget, bool *pcreated, int uid, int gid);
   void encode_cap_releases(MetaRequest *request, int mds);
   int encode_inode_release(Inode *in, MetaRequest *req,
 			   int mds, int drop,
@@ -258,9 +264,9 @@ public:
 			     int mds, int drop, int unless);
   int choose_target_mds(MetaRequest *req);
   void connect_mds_targets(int mds);
-  void send_request(MetaRequest *request, int mds);
+  void send_request(MetaRequest *request, MetaSession *session);
   MClientRequest *build_client_request(MetaRequest *request);
-  void kick_requests(int mds, bool signal);
+  void kick_requests(MetaSession *session, bool signal);
   void handle_client_request_forward(MClientRequestForward *reply);
   void handle_client_reply(MClientReply *reply);
 
@@ -332,7 +338,7 @@ protected:
   Mutex                  client_lock;
 
   // helpers
-  void wake_inode_waiters(int mds);
+  void wake_inode_waiters(MetaSession *s);
   void wait_on_list(list<Cond*>& ls);
   void signal_cond_list(list<Cond*>& ls);
 
@@ -365,7 +371,7 @@ protected:
   // trim cache.
   void trim_cache();
   void trim_dentry(Dentry *dn);
-  void trim_caps(int mds, int max);
+  void trim_caps(MetaSession *s, int max);
   
   void dump_inode(Formatter *f, Inode *in, set<Inode*>& did, bool disconnected);
   void dump_cache(Formatter *f);  // debug
@@ -407,11 +413,9 @@ protected:
 
   void handle_lease(MClientLease *m);
 
-  void release_lease(Inode *in, Dentry *dn, int mask);
-
   // file caps
   void check_cap_issue(Inode *in, Cap *cap, unsigned issued);
-  void add_update_cap(Inode *in, int mds, uint64_t cap_id,
+  void add_update_cap(Inode *in, MetaSession *session, uint64_t cap_id,
 		      unsigned issued, unsigned seq, unsigned mseq, inodeno_t realm,
 		      int flags);
   void remove_cap(Cap *cap);
@@ -420,8 +424,8 @@ protected:
   void mark_caps_dirty(Inode *in, int caps);
   int mark_caps_flushing(Inode *in);
   void flush_caps();
-  void flush_caps(Inode *in, int mds);
-  void kick_flushing_caps(int mds);
+  void flush_caps(Inode *in, MetaSession *session);
+  void kick_flushing_caps(MetaSession *session);
   int get_caps(Inode *in, int need, int want, int *have, loff_t endoff);
 
   void maybe_update_snaprealm(SnapRealm *realm, snapid_t snap_created, snapid_t snap_highwater, 
@@ -429,14 +433,15 @@ protected:
 
   void handle_snap(class MClientSnap *m);
   void handle_caps(class MClientCaps *m);
-  void handle_cap_import(Inode *in, class MClientCaps *m);
-  void handle_cap_export(Inode *in, class MClientCaps *m);
-  void handle_cap_trunc(Inode *in, class MClientCaps *m);
-  void handle_cap_flush_ack(Inode *in, int mds, Cap *cap, class MClientCaps *m);
-  void handle_cap_flushsnap_ack(Inode *in, class MClientCaps *m);
-  void handle_cap_grant(Inode *in, int mds, Cap *cap, class MClientCaps *m);
+  void handle_cap_import(MetaSession *session, Inode *in, class MClientCaps *m);
+  void handle_cap_export(MetaSession *session, Inode *in, class MClientCaps *m);
+  void handle_cap_trunc(MetaSession *session, Inode *in, class MClientCaps *m);
+  void handle_cap_flush_ack(MetaSession *session, Inode *in, Cap *cap, class MClientCaps *m);
+  void handle_cap_flushsnap_ack(MetaSession *session, Inode *in, class MClientCaps *m);
+  void handle_cap_grant(MetaSession *session, Inode *in, Cap *cap, class MClientCaps *m);
   void cap_delay_requeue(Inode *in);
-  void send_cap(Inode *in, int mds, Cap *cap, int used, int want, int retain, int flush);
+  void send_cap(Inode *in, MetaSession *session, Cap *cap,
+		int used, int want, int retain, int flush);
   void check_caps(Inode *in, bool is_delayed);
   void get_cap_ref(Inode *in, int cap);
   void put_cap_ref(Inode *in, int cap);
@@ -451,7 +456,19 @@ protected:
   void _invalidate_inode_cache(Inode *in, int64_t off, int64_t len, bool keep_caps);
   void _async_invalidate(Inode *in, int64_t off, int64_t len, bool keep_caps);
   void _release(Inode *in);
-  bool _flush(Inode *in);
+  
+  /**
+   * Initiate a flush of the data associated with the given inode.
+   * If you specify a Context, you are responsible for holding an inode
+   * reference for the duration of the flush. If not, _flush() will
+   * take the reference for you.
+   * @param in The Inode whose data you wish to flush.
+   * @param c The Context you wish us to complete once the data is
+   * flushed. If already flushed, this will be called in-line.
+   * 
+   * @returns true if the data was already flushed, false otherwise.
+   */
+  bool _flush(Inode *in, Context *c=NULL);
   void _flush_range(Inode *in, int64_t off, uint64_t size);
   void _flushed(Inode *in);
   void flush_set_callback(ObjectCacher::ObjectSet *oset);
@@ -465,17 +482,17 @@ protected:
   // metadata cache
   void update_dir_dist(Inode *in, DirStat *st);
 
-  void insert_readdir_results(MetaRequest *request, int mds, Inode *diri);
-  Inode* insert_trace(MetaRequest *request, int mds);
+  void insert_readdir_results(MetaRequest *request, MetaSession *session, Inode *diri);
+  Inode* insert_trace(MetaRequest *request, MetaSession *session);
   void update_inode_file_bits(Inode *in,
 			      uint64_t truncate_seq, uint64_t truncate_size, uint64_t size,
 			      uint64_t time_warp_seq, utime_t ctime, utime_t mtime, utime_t atime,
 			      int issued);
-  Inode *add_update_inode(InodeStat *st, utime_t ttl, int mds);
+  Inode *add_update_inode(InodeStat *st, utime_t ttl, MetaSession *session);
   Dentry *insert_dentry_inode(Dir *dir, const string& dname, LeaseStat *dlease, 
-			      Inode *in, utime_t from, int mds, bool set_offset,
+			      Inode *in, utime_t from, MetaSession *session, bool set_offset,
 			      Dentry *old_dentry = NULL);
-  void update_dentry_lease(Dentry *dn, LeaseStat *dlease, utime_t from, int mds);
+  void update_dentry_lease(Dentry *dn, LeaseStat *dlease, utime_t from, MetaSession *session);
 
 
   // ----------------------
@@ -509,18 +526,18 @@ private:
 
   // internal interface
   //   call these with client_lock held!
-  int _do_lookup(Inode *dir, const char *name, Inode **target);
+  int _do_lookup(Inode *dir, const string& name, Inode **target);
   int _lookup(Inode *dir, const string& dname, Inode **target);
 
-  int _link(Inode *in, Inode *dir, const char *name, int uid=-1, int gid=-1);
+  int _link(Inode *in, Inode *dir, const char *name, int uid=-1, int gid=-1, Inode **inp = 0);
   int _unlink(Inode *dir, const char *name, int uid=-1, int gid=-1);
   int _rename(Inode *olddir, const char *oname, Inode *ndir, const char *nname, int uid=-1, int gid=-1);
-  int _mkdir(Inode *dir, const char *name, mode_t mode, int uid=-1, int gid=-1);
+  int _mkdir(Inode *dir, const char *name, mode_t mode, int uid=-1, int gid=-1, Inode **inp = 0);
   int _rmdir(Inode *dir, const char *name, int uid=-1, int gid=-1);
-  int _symlink(Inode *dir, const char *name, const char *target, int uid=-1, int gid=-1);
-  int _mknod(Inode *dir, const char *name, mode_t mode, dev_t rdev, int uid=-1, int gid=-1);
-  int _setattr(Inode *in, struct stat *attr, int mask, int uid=-1, int gid=-1);
-  int _getattr(Inode *in, int mask, int uid=-1, int gid=-1);
+  int _symlink(Inode *dir, const char *name, const char *target, int uid=-1, int gid=-1, Inode **inp = 0);
+  int _mknod(Inode *dir, const char *name, mode_t mode, dev_t rdev, int uid=-1, int gid=-1, Inode **inp = 0);
+  int _setattr(Inode *in, struct stat *attr, int mask, int uid=-1, int gid=-1, Inode **inp = 0);
+  int _getattr(Inode *in, int mask, int uid=-1, int gid=-1, bool force=false);
   int _getxattr(Inode *in, const char *name, void *value, size_t len, int uid=-1, int gid=-1);
   int _listxattr(Inode *in, char *names, size_t len, int uid=-1, int gid=-1);
   int _setxattr(Inode *in, const char *name, const void *value, size_t len, int flags, int uid=-1, int gid=-1);
@@ -608,12 +625,15 @@ public:
   int lstatlite(const char *path, struct statlite *buf);
 
   int setattr(const char *relpath, struct stat *attr, int mask);
+  int fsetattr(int fd, struct stat *attr, int mask);
   int chmod(const char *path, mode_t mode);
   int fchmod(int fd, mode_t mode);
-  int chown(const char *path, uid_t uid, gid_t gid);
-  int fchown(int fd, uid_t uid, gid_t gid);
-  int lchown(const char *path, uid_t uid, gid_t gid);
+  int lchmod(const char *path, mode_t mode);
+  int chown(const char *path, int uid, int gid);
+  int fchown(int fd, int uid, int gid);
+  int lchown(const char *path, int uid, int gid);
   int utime(const char *path, struct utimbuf *buf);
+  int lutime(const char *path, struct utimbuf *buf);
   int truncate(const char *path, loff_t size);
 
   // file ops
@@ -651,12 +671,15 @@ public:
   // expose file layout
   int describe_layout(int fd, ceph_file_layout* layout);
   int get_file_stripe_address(int fd, loff_t offset, vector<entity_addr_t>& address);
+  int get_file_extent_osds(int fd, loff_t off, loff_t *len, vector<int>& osds);
+  int get_osd_addr(int osd, entity_addr_t& addr);
 
   // expose osdmap 
   int get_local_osd();
   int get_pool_replication(int64_t pool);
   int64_t get_pool_id(const char *pool_name);
   string get_pool_name(int64_t pool);
+  int get_osd_crush_location(int id, vector<pair<string, string> >& path);
 
   int enumerate_layout(int fd, vector<ObjectExtent>& result,
 		       loff_t length, loff_t offset);
