@@ -14,18 +14,18 @@ void MetaRequest::dump(Formatter *f) const
   f->dump_string("op", ceph_mds_op_name(head.op));
   f->dump_stream("path") << path;
   f->dump_stream("path2") << path2;
-  if (inode)
-    f->dump_stream("ino") << inode->ino;
-  if (old_inode)
-    f->dump_stream("old_ino") << old_inode->ino;
-  if (other_inode)
-    f->dump_stream("other_ino") << other_inode->ino;
+  if (_inode)
+    f->dump_stream("ino") << _inode->ino;
+  if (_old_inode)
+    f->dump_stream("old_ino") << _old_inode->ino;
+  if (_other_inode)
+    f->dump_stream("other_ino") << _other_inode->ino;
   if (target)
     f->dump_stream("target_ino") << target->ino;
-  if (dentry)
-    f->dump_string("dentry", dentry->name);
-  if (old_dentry)
-    f->dump_string("old_dentry", old_dentry->name);
+  if (_dentry)
+    f->dump_string("dentry", _dentry->name);
+  if (_old_dentry)
+    f->dump_string("old_dentry", _old_dentry->name);
   f->dump_stream("hint_ino") << inodeno_t(head.ino);
 
   f->dump_stream("sent_stamp") << sent_stamp;
@@ -58,11 +58,61 @@ void MetaRequest::dump(Formatter *f) const
 
 MetaRequest::~MetaRequest()
 {
-  if (dentry)
-    dentry->put();
-  if (old_dentry)
-    old_dentry->put();
+  if (_inode)
+    _inode->put();
+  if (_old_inode)
+    _old_inode->put();
+  if (_other_inode)
+    _other_inode->put();
+  if (_dentry)
+    _dentry->put();
+  if (_old_dentry)
+    _old_dentry->put();
   if (reply)
     reply->put();
 }
 
+void MetaRequest::set_inode(Inode *in) {
+  assert(_inode == NULL);
+  _inode = in;
+  _inode->get();
+}
+Inode *MetaRequest::inode() {
+  return _inode;
+}
+
+void MetaRequest::set_old_inode(Inode *in) {
+  assert(_old_inode == NULL);
+  _old_inode = in;
+  _old_inode->get();
+}
+Inode *MetaRequest::old_inode() {
+  return _old_inode;
+}
+
+void MetaRequest::set_other_inode(Inode *in) {
+  assert(_other_inode == NULL);
+  _other_inode = in;
+  _other_inode->get();
+}
+Inode *MetaRequest::other_inode() {
+  return _other_inode;
+}
+
+void MetaRequest::set_dentry(Dentry *d) {
+  assert(_dentry == NULL);
+  _dentry = d;
+  _dentry->get();
+}
+Dentry *MetaRequest::dentry() {
+  return _dentry;
+}
+
+void MetaRequest::set_old_dentry(Dentry *d) {
+  assert(_old_dentry == NULL);
+  _old_dentry = d;
+  _old_dentry->get();
+}
+Dentry *MetaRequest::old_dentry() {
+  return _old_dentry;
+}
