@@ -341,7 +341,7 @@ namespace librbd {
       map<string,bufferlist> m;
       ::decode(header, p);
       ::decode(m, p);
-      for (map<string,bufferlist>::iterator q = m.begin(); q != m.end(); q++) {
+      for (map<string,bufferlist>::iterator q = m.begin(); q != m.end(); ++q) {
 	names.push_back(q->first);
       }
     }
@@ -376,7 +376,7 @@ namespace librbd {
       return r;
 
     // no children for non-layered or old format image
-    if (!ictx->features & RBD_FEATURE_LAYERING)
+    if ((ictx->features & RBD_FEATURE_LAYERING) == 0)
       return 0;
 
     parent_spec parent_spec(ictx->md_ctx.get_id(), ictx->id, ictx->snap_id);
@@ -603,9 +603,8 @@ namespace librbd {
     Rados rados(ictx->md_ctx);
     std::list<std::string> pools;
     rados.pool_list(pools);
-    std::list<std::string>::const_iterator it;
     std::set<std::string> children;
-    for (it = pools.begin(); it != pools.end(); it++) {
+    for (std::list<std::string>::const_iterator it = pools.begin(); it != pools.end(); ++it) {
       IoCtx pool_ioctx;
       r = rados.ioctx_create(it->c_str(), pool_ioctx);
       if (r < 0) {
@@ -1899,6 +1898,12 @@ reprotect_and_return_err:
     // ignore return value, since we may be set to a non-existent
     // snapshot and the user is trying to fix that
     ictx_check(ictx);
+    if (ictx->object_cacher) {
+      // complete pending writes before we're set to a snapshot and
+      // get -EROFS for writes
+      RWLock::WLocker l(ictx->md_lock);
+      ictx->flush_cache();
+    }
     return _snap_set(ictx, snap_name);
   }
 
@@ -2436,6 +2441,12 @@ reprotect_and_return_err:
     req->complete(rados_aio_get_return_value(c));
   }
 
+  void rados_ctx_cb(rados_completion_t c, void *arg)
+  {
+    Context *comp = reinterpret_cast<Context *>(arg);
+    comp->complete(rados_aio_get_return_value(c));
+  }
+
   // validate extent against image size; clip to image size if necessary
   int clip_io(ImageCtx *ictx, uint64_t off, uint64_t *len)
   {
@@ -2464,6 +2475,36 @@ reprotect_and_return_err:
     return 0;
   }
 
+  int aio_flush(ImageCtx *ictx, AioCompletion *c)
+  {
+    CephContext *cct = ictx->cct;
+    ldout(cct, 20) << "aio_flush " << ictx << " completion " << c <<  dendl;
+
+    int r = ictx_check(ictx);
+    if (r < 0)
+      return r;
+
+    ictx->user_flushed();
+
+    c->get();
+    c->add_request();
+    c->init_time(ictx, AIO_TYPE_FLUSH);
+    C_AioWrite *req_comp = new C_AioWrite(cct, c);
+    if (ictx->object_cacher) {
+      ictx->flush_cache_aio(req_comp);
+    } else {
+      librados::AioCompletion *rados_completion =
+	librados::Rados::aio_create_completion(req_comp, NULL, rados_ctx_cb);
+      ictx->data_ctx.aio_flush_async(rados_completion);
+      rados_completion->release();
+    }
+    c->finish_adding_requests(cct);
+    c->put();
+    ictx->perfcounter->inc(l_librbd_aio_flush);
+
+    return 0;
+  }
+
   int flush(ImageCtx *ictx)
   {
     CephContext *cct = ictx->cct;
@@ -2473,6 +2514,7 @@ reprotect_and_return_err:
     if (r < 0)
       return r;
 
+    ictx->user_flushed();
     return _flush(ictx);
   }
 
@@ -2544,9 +2586,10 @@ reprotect_and_return_err:
 	bl.append(buf + q->first, q->second);
       }
 
+      C_AioWrite *req_comp = new C_AioWrite(cct, c);
       if (ictx->object_cacher) {
-	// may block
-	ictx->write_to_cache(p->oid, bl, p->length, p->offset);
+	c->add_request();
+	ictx->write_to_cache(p->oid, bl, p->length, p->offset, req_comp);
       } else {
 	// reverse map this object extent onto the parent
 	vector<pair<uint64_t,uint64_t> > objectx;
@@ -2555,7 +2598,6 @@ reprotect_and_return_err:
 			      objectx);
 	uint64_t object_overlap = ictx->prune_parent_extents(objectx, overlap);
 
-	C_AioWrite *req_comp = new C_AioWrite(cct, c);
 	AioWrite *req = new AioWrite(ictx, p->oid.name, p->objectno, p->offset,
 				     objectx, object_overlap,
 				     bl, snapc, snap_id, req_comp);
