@@ -127,7 +127,6 @@ ostream& operator<<(ostream& out, CInode& in)
   if (in.state_test(CInode::STATE_AMBIGUOUSAUTH)) out << " AMBIGAUTH";
   if (in.state_test(CInode::STATE_NEEDSRECOVER)) out << " needsrecover";
   if (in.state_test(CInode::STATE_RECOVERING)) out << " recovering";
-  if (in.state_test(CInode::STATE_DIRTYPARENT)) out << " dirtyparent";
   if (in.is_freezing_inode()) out << " FREEZING=" << in.auth_pin_freeze_allowance;
   if (in.is_frozen_inode()) out << " FROZEN";
   if (in.is_frozen_auth_pin()) out << " FROZEN_AUTHPIN";
@@ -202,7 +201,7 @@ ostream& operator<<(ostream& out, CInode& in)
     out << " caps={";
     for (map<client_t,Capability*>::iterator it = in.get_client_caps().begin();
          it != in.get_client_caps().end();
-         it++) {
+         ++it) {
       if (it != in.get_client_caps().begin()) out << ",";
       out << it->first << "="
 	  << ccap_string(it->second->pending());
@@ -313,14 +312,11 @@ inode_t *CInode::project_inode(map<string,bufferptr> *px)
     projected_nodes.push_back(new projected_inode_t(new inode_t(inode)));
     if (px)
       *px = xattrs;
-    projected_nodes.back()->dir_layout = default_layout;
   } else {
-    default_file_layout *last_dl = projected_nodes.back()->dir_layout;
     projected_nodes.push_back(new projected_inode_t(
         new inode_t(*projected_nodes.back()->inode)));
     if (px)
       *px = *get_projected_xattrs();
-    projected_nodes.back()->dir_layout = last_dl;
   }
   projected_nodes.back()->xattrs = px;
   dout(15) << "project_inode " << projected_nodes.back()->inode << dendl;
@@ -341,11 +337,6 @@ void CInode::pop_and_dirty_projected_inode(LogSegment *ls)
     delete px;
   }
 
-  if (projected_nodes.front()->dir_layout != default_layout) {
-    delete default_layout;
-    default_layout = projected_nodes.front()->dir_layout;
-  }
-
   if (projected_nodes.front()->snapnode)
     pop_projected_snaprealm(projected_nodes.front()->snapnode);
 
@@ -760,13 +751,13 @@ void CInode::make_path_string_projected(string& s)
 {
   make_path_string(s);
   
-  if (projected_parent.size()) {
+  if (!projected_parent.empty()) {
     string q;
     q.swap(s);
     s = "{" + q;
     for (list<CDentry*>::iterator p = projected_parent.begin();
 	 p != projected_parent.end();
-	 p++) {
+	 ++p) {
       string q;
       make_path_string(q, true, *p);
       s += " ";
@@ -805,7 +796,7 @@ void CInode::name_stray_dentry(string& dname)
 version_t CInode::pre_dirty()
 {
   version_t pv; 
-  if (parent || projected_parent.size()) {
+  if (parent || !projected_parent.empty()) {
     pv = get_projected_parent_dn()->pre_dirty(get_projected_version());
     dout(10) << "pre_dirty " << pv << " (current v " << inode.version << ")" << dendl;
   } else {
@@ -975,90 +966,106 @@ void CInode::_fetched(bufferlist& bl, bufferlist& bl2, Context *fin)
   delete fin;
 }
 
+class C_CInode_FetchedBacktrace : public Context {
+  CInode *in;
+  inode_backtrace_t *backtrace;
+  Context *fin;
+public:
+  bufferlist bl;
+  C_CInode_FetchedBacktrace(CInode *i, inode_backtrace_t *bt, Context *f) :
+    in(i), backtrace(bt), fin(f) {}
 
+  void finish(int r) {
+    if (r == 0) {
+      in->_fetched_backtrace(&bl, backtrace, fin);
+    } else {
+      fin->finish(r);
+    }
+  }
+};
 
-// ------------------
-// parent dir
-
-void CInode::build_backtrace(inode_backtrace_t& bt)
+void CInode::fetch_backtrace(inode_backtrace_t *bt, Context *fin)
 {
-  bt.ino = inode.ino;
-  bt.ancestors.clear();
+  object_t oid = get_object_name(ino(), frag_t(), "");
+  object_locator_t oloc(inode.layout.fl_pg_pool);
+
+  SnapContext snapc;
+  C_CInode_FetchedBacktrace *c = new C_CInode_FetchedBacktrace(this, bt, fin);
+  mdcache->mds->objecter->getxattr(oid, oloc, "parent", CEPH_NOSNAP, &c->bl, 0, c);
+}
+
+void CInode::_fetched_backtrace(bufferlist *bl, inode_backtrace_t *bt, Context *fin)
+{
+  ::decode(*bt, *bl);
+  if (fin) {
+    fin->finish(0);
+  }
+}
+
+void CInode::build_backtrace(int64_t location, inode_backtrace_t* bt)
+{
+  bt->ino = inode.ino;
+  bt->ancestors.clear();
 
   CInode *in = this;
   CDentry *pdn = get_parent_dn();
   while (pdn) {
     CInode *diri = pdn->get_dir()->get_inode();
-    bt.ancestors.push_back(inode_backpointer_t(diri->ino(), pdn->name, in->inode.version));
+    bt->ancestors.push_back(inode_backpointer_t(diri->ino(), pdn->name, in->inode.version));
     in = diri;
     pdn = in->get_parent_dn();
   }
-}
-
-unsigned CInode::encode_parent_mutation(ObjectOperation& m)
-{
-  string path;
-  make_path_string(path);
-  m.setxattr("path", path);
-
-  inode_backtrace_t bt;
-  build_backtrace(bt);
-  
-  bufferlist parent;
-  ::encode(bt, parent);
-  m.setxattr("parent", parent);
-  return path.length() + parent.length();
-}
-
-struct C_Inode_StoredParent : public Context {
-  CInode *in;
-  version_t version;
-  Context *fin;
-  C_Inode_StoredParent(CInode *i, version_t v, Context *f) : in(i), version(v), fin(f) {}
-  void finish(int r) {
-    in->_stored_parent(version, fin);
-  }
-};
-
-void CInode::store_parent(Context *fin)
-{
-  dout(10) << "store_parent" << dendl;
-  
-  ObjectOperation m;
-  encode_parent_mutation(m);
-
-  // write it.
-  SnapContext snapc;
-
-  object_t oid = get_object_name(ino(), frag_t(), "");
-  object_locator_t oloc(mdcache->mds->mdsmap->get_metadata_pool());
-
-  mdcache->mds->objecter->mutate(oid, oloc, m, snapc, ceph_clock_now(g_ceph_context), 0,
-				 NULL, new C_Inode_StoredParent(this, inode.last_renamed_version, fin) );
-
-}
-
-void CInode::_stored_parent(version_t v, Context *fin)
-{
-  if (state_test(STATE_DIRTYPARENT)) {
-    if (v == inode.last_renamed_version) {
-      dout(10) << "stored_parent committed v" << v << ", removing from list" << dendl;
-      item_renamed_file.remove_myself();
-      state_clear(STATE_DIRTYPARENT);
-      put(PIN_DIRTYPARENT);
-    } else {
-      dout(10) << "stored_parent committed v" << v << " < " << inode.last_renamed_version
-	       << ", renamed again, not removing from list" << dendl;
+  vector<int64_t>::iterator i = inode.old_pools.begin();
+  while(i != inode.old_pools.end()) {
+    // don't add our own pool id to old_pools to avoid looping (e.g. setlayout 0, 1, 0)
+    if (*i == location) {
+      ++i;
+      continue;
     }
-  } else {
-    dout(10) << "stored_parent committed v" << v << ", tho i wasn't on the renamed_files list" << dendl;
-  }
-  if (fin) {
-    fin->finish(0);
-    delete fin;
+    bt->old_pools.insert(*i);
+    ++i;
   }
 }
 
+// ------------------
+// parent dir
+
+void CInode::encode_store(bufferlist& bl)
+{
+  ENCODE_START(4, 4, bl);
+  ::encode(inode, bl);
+  if (is_symlink())
+    ::encode(symlink, bl);
+  ::encode(dirfragtree, bl);
+  ::encode(xattrs, bl);
+  bufferlist snapbl;
+  encode_snap_blob(snapbl);
+  ::encode(snapbl, bl);
+  ::encode(old_inodes, bl);
+  ENCODE_FINISH(bl);
+}
+
+void CInode::decode_store(bufferlist::iterator& bl) {
+  DECODE_START_LEGACY_COMPAT_LEN(4, 4, 4, bl);
+  ::decode(inode, bl);
+  if (is_symlink())
+    ::decode(symlink, bl);
+  ::decode(dirfragtree, bl);
+  ::decode(xattrs, bl);
+  bufferlist snapbl;
+  ::decode(snapbl, bl);
+  decode_snap_blob(snapbl);
+  ::decode(old_inodes, bl);
+  if (struct_v == 2 && inode.is_dir()) {
+    bool default_layout_exists;
+    ::decode(default_layout_exists, bl);
+    if (default_layout_exists) {
+      ::decode(struct_v, bl); // this was a default_file_layout
+      ::decode(inode.layout, bl); // but we only care about the layout portion
+    }
+  }
+  DECODE_FINISH(bl);
+}
 
 // ------------------
 // locking
@@ -1111,12 +1118,14 @@ void CInode::encode_lock_state(int type, bufferlist& bl)
     
   case CEPH_LOCK_IFILE:
     if (is_auth()) {
-      ::encode(inode.layout, bl);
-      ::encode(inode.size, bl);
       ::encode(inode.mtime, bl);
       ::encode(inode.atime, bl);
       ::encode(inode.time_warp_seq, bl);
-      ::encode(inode.client_ranges, bl);
+      if (!is_dir()) {
+	::encode(inode.layout, bl);
+	::encode(inode.size, bl);
+	::encode(inode.client_ranges, bl);
+      }
     } else {
       bool dirty = filelock.is_dirty();
       ::encode(dirty, bl);
@@ -1198,9 +1207,7 @@ void CInode::encode_lock_state(int type, bufferlist& bl)
 
   case CEPH_LOCK_IPOLICY:
     if (inode.is_dir()) {
-      ::encode((default_layout ? true : false), bl);
-      if (default_layout)
-        encode(*default_layout, bl);
+      ::encode(inode.layout, bl);
     }
     break;
   
@@ -1281,7 +1288,7 @@ void CInode::decode_lock_state(int type, bufferlist& bl)
 	dirfragtree.swap(temp);
 	for (map<frag_t,CDir*>::iterator p = dirfrags.begin();
 	     p != dirfrags.end();
-	     p++)
+	     ++p)
 	  if (!dirfragtree.is_leaf(p->first)) {
 	    dout(10) << " forcing open dirfrag " << p->first << " to leaf (racing with split|merge)" << dendl;
 	    dirfragtree.force_to_leaf(g_ceph_context, p->first);
@@ -1294,12 +1301,14 @@ void CInode::decode_lock_state(int type, bufferlist& bl)
 
   case CEPH_LOCK_IFILE:
     if (!is_auth()) {
-      ::decode(inode.layout, p);
-      ::decode(inode.size, p);
       ::decode(inode.mtime, p);
       ::decode(inode.atime, p);
       ::decode(inode.time_warp_seq, p);
-      ::decode(inode.client_ranges, p);
+      if (!is_dir()) {
+	::decode(inode.layout, p);
+	::decode(inode.size, p);
+	::decode(inode.client_ranges, p);
+      }
     } else {
       bool replica_dirty;
       ::decode(replica_dirty, p);
@@ -1401,7 +1410,7 @@ void CInode::decode_lock_state(int type, bufferlist& bl)
 	  dir->fnode.rstat = rstat;
 	  dir->fnode.accounted_rstat = accounted_rstat;
 	  dir->dirty_old_rstat.swap(dirty_old_rstat);
-	  if (!(rstat == accounted_rstat) || dir->dirty_old_rstat.size()) {
+	  if (!(rstat == accounted_rstat) || !dir->dirty_old_rstat.empty()) {
 	    dout(10) << fg << " setting nestlock updated flag" << dendl;
 	    nestlock.mark_dirty();  // ok bc we're auth and caller will handle
 	  }
@@ -1441,13 +1450,7 @@ void CInode::decode_lock_state(int type, bufferlist& bl)
 
   case CEPH_LOCK_IPOLICY:
     if (inode.is_dir()) {
-      bool default_layout_exists;
-      ::decode(default_layout_exists, p);
-      if (default_layout_exists) {
-        delete default_layout;
-        default_layout = new default_file_layout;
-        decode(*default_layout, p);
-      }
+      ::decode(inode.layout, p);
     }
     break;
 
@@ -1507,7 +1510,7 @@ void CInode::start_scatter(ScatterLock *lock)
 
   for (map<frag_t,CDir*>::iterator p = dirfrags.begin();
        p != dirfrags.end();
-       p++) {
+       ++p) {
     frag_t fg = p->first;
     CDir *dir = p->second;
     fnode_t *pf = dir->get_projected_fnode();
@@ -1639,7 +1642,7 @@ void CInode::finish_scatter_gather_update(int type)
       pi->dirstat.version++;
       for (map<frag_t,CDir*>::iterator p = dirfrags.begin();
 	   p != dirfrags.end();
-	   p++) {
+	   ++p) {
 	frag_t fg = p->first;
 	CDir *dir = p->second;
 	dout(20) << fg << " " << *dir << dendl;
@@ -1720,7 +1723,7 @@ void CInode::finish_scatter_gather_update(int type)
       pi->rstat.version++;
       for (map<frag_t,CDir*>::iterator p = dirfrags.begin();
 	   p != dirfrags.end();
-	   p++) {
+	   ++p) {
 	frag_t fg = p->first;
 	CDir *dir = p->second;
 	dout(20) << fg << " " << *dir << dendl;
@@ -1745,7 +1748,7 @@ void CInode::finish_scatter_gather_update(int type)
 					       dir->first, CEPH_NOSNAP, this, true);
 	  for (map<snapid_t,old_rstat_t>::iterator q = dir->dirty_old_rstat.begin();
 	       q != dir->dirty_old_rstat.end();
-	       q++)
+	       ++q)
 	    mdcache->project_rstat_frag_to_inode(q->second.rstat, q->second.accounted_rstat,
 						 q->second.first, q->first, this, true);
 	  if (update)  // dir contents not valid if frozen or non-auth
@@ -1810,7 +1813,7 @@ void CInode::finish_scatter_gather_update_accounted(int type, Mutation *mut, EMe
 
   for (map<frag_t,CDir*>::iterator p = dirfrags.begin();
        p != dirfrags.end();
-       p++) {
+       ++p) {
     CDir *dir = p->second;
     if (!dir->is_auth() || dir->is_frozen())
       continue;
@@ -2119,7 +2122,7 @@ void CInode::purge_stale_snap_data(const set<snapid_t>& snaps)
       dout(10) << " purging old_inode [" << p->second.first << "," << p->first << "]" << dendl;
       old_inodes.erase(p++);
     } else
-      p++;
+      ++p;
   }
 }
 
@@ -2225,7 +2228,7 @@ client_t CInode::calc_ideal_loner()
   client_t loner = -1;
   for (map<client_t,Capability*>::iterator it = client_caps.begin();
        it != client_caps.end();
-       it++) 
+       ++it) 
     if (!it->second->is_stale() &&
 	((it->second->wanted() & (CEPH_CAP_ANY_WR|CEPH_CAP_FILE_WR|CEPH_CAP_FILE_RD)) ||
 	 (inode.is_dir() && !has_subtree_root_dirfrag()))) {
@@ -2387,7 +2390,7 @@ void CInode::move_to_realm(SnapRealm *realm)
 	   << ", leaving realm " << *containing_realm << dendl;
   for (map<client_t,Capability*>::iterator q = client_caps.begin();
        q != client_caps.end();
-       q++) {
+       ++q) {
     containing_realm->remove_cap(q->first, q->second);
     realm->add_cap(q->first, q->second);
   }
@@ -2426,7 +2429,7 @@ void CInode::export_client_caps(map<client_t,Capability::Export>& cl)
 {
   for (map<client_t,Capability*>::iterator it = client_caps.begin();
        it != client_caps.end();
-       it++) {
+       ++it) {
     cl[it->first] = it->second->make_export();
   }
 }
@@ -2507,7 +2510,7 @@ int CInode::get_caps_issued(int *ploner, int *pother, int *pxlocker,
     loner_cap = -1;
   for (map<client_t,Capability*>::iterator it = client_caps.begin();
        it != client_caps.end();
-       it++) {
+       ++it) {
     int i = it->second->issued();
     c |= i;
     if (it->first == loner_cap)
@@ -2526,7 +2529,7 @@ bool CInode::is_any_caps_wanted()
 {
   for (map<client_t,Capability*>::iterator it = client_caps.begin();
        it != client_caps.end();
-       it++)
+       ++it)
     if (it->second->wanted())
       return true;
   return false;
@@ -2538,7 +2541,7 @@ int CInode::get_caps_wanted(int *ploner, int *pother, int shift, int mask)
   int loner = 0, other = 0;
   for (map<client_t,Capability*>::iterator it = client_caps.begin();
        it != client_caps.end();
-       it++) {
+       ++it) {
     if (!it->second->is_stale()) {
       int t = it->second->wanted();
       w |= t;
@@ -2552,7 +2555,7 @@ int CInode::get_caps_wanted(int *ploner, int *pother, int shift, int mask)
   if (is_auth())
     for (map<int,int>::iterator it = mds_caps_wanted.begin();
 	 it != mds_caps_wanted.end();
-	 it++) {
+	 ++it) {
       w |= it->second;
       other |= it->second;
       //cout << " get_caps_wanted mds " << it->first << " " << cap_string(it->second) << endl;
@@ -2599,7 +2602,7 @@ int CInode::encode_inodestat(bufferlist& bl, Session *session,
 			      SnapRealm *dir_realm,
 			      snapid_t snapid, unsigned max_bytes)
 {
-  int client = session->inst.name.num();
+  int client = session->info.inst.name.num();
   assert(snapid);
   assert(session->connection);
   
@@ -2677,16 +2680,10 @@ int CInode::encode_inodestat(bufferlist& bl, Session *session,
 
   // file
   i = pfile ? pi:oi;
-  if (is_file()) {
-    e.layout = i->layout;
-  } else if (is_dir()) {
-    ceph_file_layout *l = ppolicy ? get_projected_dir_layout() : ( default_layout ? &default_layout->layout : NULL );
-    if (l)
-      e.layout = *l;
-    else
-      memset(&e.layout, 0, sizeof(e.layout));
+  if (is_dir()) {
+    e.layout = (ppolicy ? pi : oi)->layout;
   } else {
-    memset(&e.layout, 0, sizeof(e.layout));
+    e.layout = i->layout;
   }
   e.size = i->size;
   e.truncate_seq = i->truncate_seq;
@@ -2825,7 +2822,7 @@ int CInode::encode_inodestat(bufferlist& bl, Session *session,
   ::encode(e, bl);
   for (map<frag_t,int32_t>::iterator p = dirfragtree._splits.begin();
        p != dirfragtree._splits.end();
-       p++) {
+       ++p) {
     ::encode(p->first, bl);
     ::encode(p->second, bl);
   }
@@ -2991,8 +2988,7 @@ void CInode::_decode_locks_rejoin(bufferlist::iterator& p, list<Context*>& waite
 
 void CInode::encode_export(bufferlist& bl)
 {
-  __u8 struct_v = 2;
-  ::encode(struct_v, bl);
+  ENCODE_START(3, 3, bl)
   _encode_base(bl);
 
   bool dirty = is_dirty();
@@ -3022,6 +3018,7 @@ void CInode::encode_export(bufferlist& bl)
 
   _encode_locks_full(bl);
   get(PIN_TEMPEXPORTING);
+  ENCODE_FINISH(bl);
 }
 
 void CInode::finish_export(utime_t now)
@@ -3039,8 +3036,7 @@ void CInode::finish_export(utime_t now)
 void CInode::decode_import(bufferlist::iterator& p,
 			   LogSegment *ls)
 {
-  __u8 struct_v;
-  ::decode(struct_v, p);
+  DECODE_START_LEGACY_COMPAT_LEN(3, 3, 3, p);
 
   _decode_base(p);
 
@@ -3102,4 +3098,5 @@ void CInode::decode_import(bufferlist::iterator& p,
   }
 
   _decode_locks_full(p);
+  DECODE_FINISH(p);
 }
