@@ -322,7 +322,7 @@ public:
       return -1;
     map<int32_t,uint64_t>::iterator p = up.begin();
     for (int n = rand() % up.size(); n; n--)
-      p++;
+      ++p;
     return p->first;
   }
 
@@ -410,7 +410,12 @@ public:
   bool is_clientreplay(int m)   { return get_state(m) == STATE_CLIENTREPLAY; }
   bool is_active(int m)   { return get_state(m) == STATE_ACTIVE; }
   bool is_stopping(int m) { return get_state(m) == STATE_STOPPING; }
-  bool is_clientreplay_or_active_or_stopping(int m)   { return is_clientreplay(m) || is_active(m) || is_stopping(m); }
+  bool is_active_or_stopping(int m) {
+    return is_active(m) || is_stopping(m);
+  }
+  bool is_clientreplay_or_active_or_stopping(int m) {
+    return is_clientreplay(m) || is_active(m) || is_stopping(m);
+  }
 
   bool is_followable(int m) {
     return (is_resolve(m) ||
