@@ -14,9 +14,9 @@
 
 #include <sstream>
 
-#include "AuthMonitor.h"
-#include "Monitor.h"
-#include "MonitorStore.h"
+#include "mon/AuthMonitor.h"
+#include "mon/Monitor.h"
+#include "mon/MonitorDBStore.h"
 
 #include "messages/MMonCommand.h"
 #include "messages/MAuth.h"
@@ -33,10 +33,11 @@
 
 #include "common/config.h"
 #include "include/assert.h"
+#include <sstream>
 
 #define dout_subsys ceph_subsys_mon
 #undef dout_prefix
-#define dout_prefix _prefix(_dout, mon, paxos->get_version())
+#define dout_prefix _prefix(_dout, mon, get_version())
 static ostream& _prefix(std::ostream *_dout, Monitor *mon, version_t v) {
   return *_dout << "mon." << mon->name << "@" << mon->rank
 		<< "(" << mon->get_state_name()
@@ -45,8 +46,6 @@ static ostream& _prefix(std::ostream *_dout, Monitor *mon, version_t v) {
 
 ostream& operator<<(ostream& out, AuthMonitor& pm)
 {
-  std::stringstream ss;
- 
   return out << "auth";
 }
 
@@ -67,8 +66,7 @@ void AuthMonitor::check_rotate()
 
 void AuthMonitor::tick() 
 {
-  if (!paxos->is_active() ||
-      !mon->is_all_paxos_recovered()) return;
+  if (!is_active()) return;
 
   update_from_paxos();
   dout(10) << *this << dendl;
@@ -96,7 +94,8 @@ void AuthMonitor::create_initial()
 
   KeyRing keyring;
   bufferlist bl;
-  mon->store->get_bl_ss_safe(bl, "mkfs", "keyring");
+  int ret = mon->store->get("mkfs", "keyring", bl);
+  assert(ret == 0);
   bufferlist::iterator p = bl.begin();
   ::decode(keyring, p);
 
@@ -112,30 +111,42 @@ void AuthMonitor::create_initial()
 
 void AuthMonitor::update_from_paxos()
 {
-  dout(10) << "update_from_paxos()" << dendl;
-  version_t paxosv = paxos->get_version();
+  dout(10) << __func__ << dendl;
+  version_t version = get_version();
   version_t keys_ver = mon->key_server.get_ver();
-  if (paxosv == keys_ver)
+  if (version == keys_ver)
     return;
-  assert(paxosv >= keys_ver);
+  assert(version >= keys_ver);
 
-  if (keys_ver != paxos->get_stashed_version()) {
-    bufferlist latest;
-    keys_ver = paxos->get_stashed(latest);
-    dout(7) << "update_from_paxos loading summary e" << keys_ver << dendl;
-    bufferlist::iterator p = latest.begin();
+  version_t latest_full = get_version_latest_full();
+
+  dout(10) << __func__ << " version " << version << " keys ver " << keys_ver
+           << " latest " << latest_full << dendl;
+
+  if ((latest_full > 0) && (latest_full > keys_ver)) {
+    bufferlist latest_bl;
+    int err = get_version_full(latest_full, latest_bl);
+    assert(err == 0);
+    assert(latest_bl.length() != 0);
+    dout(7) << __func__ << " loading summary e " << latest_full << dendl;
+    dout(7) << __func__ << " latest length " << latest_bl.length() << dendl;
+    bufferlist::iterator p = latest_bl.begin();
     __u8 struct_v;
     ::decode(struct_v, p);
     ::decode(max_global_id, p);
     ::decode(mon->key_server, p);
-    mon->key_server.set_ver(keys_ver);
-  } 
+    mon->key_server.set_ver(latest_full);
+    keys_ver = latest_full;
+  }
+
+  dout(10) << __func__ << " key server version " << mon->key_server.get_ver() << dendl;
 
   // walk through incrementals
-  while (paxosv > keys_ver) {
+  while (version > keys_ver) {
     bufferlist bl;
-    bool success = paxos->read(keys_ver+1, bl);
-    assert(success);
+    int ret = get_version(keys_ver+1, bl);
+    assert(ret == 0);
+    assert(bl.length());
 
     // reset if we are moving to initial state.  we will normally have
     // keys in here temporarily for bootstrapping that we need to
@@ -143,6 +154,9 @@ void AuthMonitor::update_from_paxos()
     if (keys_ver == 0) 
       mon->key_server.clear_secrets();
 
+    dout(20) << __func__ << " walking through version " << (keys_ver+1)
+             << " len " << bl.length() << dendl;
+
     bufferlist::iterator p = bl.begin();
     __u8 v;
     ::decode(v, p);
@@ -169,7 +183,9 @@ void AuthMonitor::update_from_paxos()
     mon->key_server.set_ver(keys_ver);
 
     if (keys_ver == 1) {
-      mon->store->erase_ss("mkfs", "keyring");
+      MonitorDBStore::Transaction t;
+      t.erase("mkfs", "keyring");
+      mon->store->apply_transaction(t);
     }
   }
 
@@ -178,19 +194,16 @@ void AuthMonitor::update_from_paxos()
 
   dout(10) << "update_from_paxos() last_allocated_id=" << last_allocated_id
 	   << " max_global_id=" << max_global_id << dendl;
-
+ 
+  /*
   bufferlist bl;
   __u8 v = 1;
   ::encode(v, bl);
   ::encode(max_global_id, bl);
   Mutex::Locker l(mon->key_server.get_lock());
   ::encode(mon->key_server, bl);
-  paxos->stash_latest(paxosv, bl);
-
-  unsigned max = g_conf->paxos_max_join_drift * 2;
-  if (mon->is_leader() &&
-      paxosv > max)
-    paxos->trim_to(paxosv - max);
+  paxos->stash_latest(version, bl);
+  */
 }
 
 void AuthMonitor::increase_max_global_id()
@@ -207,24 +220,61 @@ void AuthMonitor::increase_max_global_id()
 
 bool AuthMonitor::should_propose(double& delay)
 {
-  return (pending_auth.size() > 0);
+  return (!pending_auth.empty());
 }
 
 void AuthMonitor::create_pending()
 {
   pending_auth.clear();
-  dout(10) << "create_pending v " << (paxos->get_version() + 1) << dendl;
+  dout(10) << "create_pending v " << (get_version() + 1) << dendl;
 }
 
-void AuthMonitor::encode_pending(bufferlist &bl)
+void AuthMonitor::encode_pending(MonitorDBStore::Transaction *t)
 {
-  dout(10) << "encode_pending v " << (paxos->get_version() + 1) << dendl;
+  dout(10) << __func__ << " v " << (get_version() + 1) << dendl;
+
+  bufferlist bl;
+
   __u8 v = 1;
   ::encode(v, bl);
-  for (vector<Incremental>::iterator p = pending_auth.begin();
-       p != pending_auth.end();
-       p++)
+  vector<Incremental>::iterator p;
+  for (p = pending_auth.begin(); p != pending_auth.end(); ++p)
     p->encode(bl, mon->get_quorum_features());
+
+  version_t version = get_version() + 1;
+  put_version(t, version, bl);
+  put_last_committed(t, version);
+}
+
+void AuthMonitor::encode_full(MonitorDBStore::Transaction *t)
+{
+  version_t version = mon->key_server.get_ver();
+  dout(10) << __func__ << " auth v " << version << dendl;
+  assert(get_version() == version);
+
+  bufferlist full_bl;
+  Mutex::Locker l(mon->key_server.get_lock());
+  if (mon->key_server.has_secrets()) {
+    dout(20) << __func__ << " key server has secrets!" << dendl;
+    __u8 v = 1;
+    ::encode(v, full_bl);
+    ::encode(max_global_id, full_bl);
+    ::encode(mon->key_server, full_bl);
+
+    put_version_full(t, version, full_bl);
+    put_version_latest_full(t, version);
+  } else {
+    dout(20) << __func__
+	     << " key server has no secrets; do not put them in tx" << dendl;
+  }
+}
+
+void AuthMonitor::update_trim()
+{
+  unsigned max = g_conf->paxos_max_join_drift * 2;
+  version_t version = get_version();
+  if (mon->is_leader() && (version > max))
+    set_trim_to(version - max);
 }
 
 bool AuthMonitor::preprocess_query(PaxosServiceMessage *m)
@@ -277,26 +327,33 @@ uint64_t AuthMonitor::assign_global_id(MAuth *m, bool should_increase_max)
 	   << " last_allocated=" << last_allocated_id << " max_global_id=" <<  max_global_id << dendl;
 
   uint64_t next_global_id = last_allocated_id + 1;
+  int remainder = next_global_id % total_mon;
+  if (remainder)
+    remainder = total_mon - remainder;
+  next_global_id += remainder + mon->rank;
+  dout(10) << "next_global_id should be " << next_global_id << dendl;
 
-  if (next_global_id < max_global_id) {
-    int remainder = next_global_id % total_mon;
-    if (remainder)
-      remainder = total_mon - remainder;
-    next_global_id += remainder + mon->rank;
-    dout(10) << "next_global_id should be " << next_global_id << dendl;
+  // if we can't bump the max, bail out now on an out-of-bounds gid
+  if (next_global_id > max_global_id &&
+      (!mon->is_leader() || !should_increase_max)) {
+    return 0;
   }
 
-  if (next_global_id >= max_global_id) {
-    if (!mon->is_leader() || !should_increase_max) {
-      return 0;
-    }
-    while (next_global_id >= max_global_id) {
-      increase_max_global_id();
-    }
+  // can we return a gid?
+  bool return_next = (next_global_id <= max_global_id);
+
+  // bump the max?
+  while (mon->is_leader() &&
+	 next_global_id >= max_global_id - g_conf->mon_globalid_prealloc / 2) {
+    increase_max_global_id();
   }
 
-  last_allocated_id = next_global_id;
-  return next_global_id;
+  if (return_next) {
+    last_allocated_id = next_global_id;
+    return next_global_id;
+  } else {
+    return 0;
+  }
 }
 
 
@@ -383,8 +440,16 @@ bool AuthMonitor::prep_auth(MAuth *m, bool paxos_writable)
   if (!s->global_id) {
     s->global_id = assign_global_id(m, paxos_writable);
     if (!s->global_id) {
+
       delete s->auth_handler;
       s->auth_handler = NULL;
+
+      if (mon->is_leader() && paxos_writable) {
+        dout(10) << "increasing global id, waitlisting message" << dendl;
+        wait_for_active(new C_RetryMessage(this, m));
+        goto done;
+      }
+
       s->put();
 
       if (!mon->is_leader()) {
@@ -393,7 +458,7 @@ bool AuthMonitor::prep_auth(MAuth *m, bool paxos_writable)
 	MMonGlobalID *req = new MMonGlobalID();
 	req->old_max_id = max_global_id;
 	mon->messenger->send_message(req, mon->monmap->get_inst(leader));
-	paxos->wait_for_commit(new C_RetryMessage(this, m));
+	wait_for_finished_proposal(new C_RetryMessage(this, m));
 	return true;
       }
 
@@ -419,7 +484,8 @@ bool AuthMonitor::prep_auth(MAuth *m, bool paxos_writable)
       ret = s->auth_handler->handle_request(indata, response_bl, s->global_id, caps_info, &auid);
     }
     if (ret == -EIO) {
-      paxos->wait_for_active(new C_RetryMessage(this, m));
+      wait_for_active(new C_RetryMessage(this,m));
+      //paxos->wait_for_active(new C_RetryMessage(this, m));
       goto done;
     }
     if (caps_info.caps.length()) {
@@ -468,7 +534,7 @@ bool AuthMonitor::preprocess_command(MMonCommand *m)
     if (!session ||
 	(!session->caps.get_allow_all() &&
 	 !mon->_allowed_command(session, m->cmd))) {
-      mon->reply_command(m, -EACCES, "access denied", rdata, paxos->get_version());
+      mon->reply_command(m, -EACCES, "access denied", rdata, get_version());
       return true;
     }
 
@@ -552,7 +618,7 @@ bool AuthMonitor::preprocess_command(MMonCommand *m)
  done:
   string rs;
   getline(ss, rs, '\0');
-  mon->reply_command(m, r, rs, rdata, paxos->get_version());
+  mon->reply_command(m, r, rs, rdata, get_version());
   return true;
 }
 
@@ -565,7 +631,7 @@ void AuthMonitor::import_keyring(KeyRing& keyring)
 {
   for (map<EntityName, EntityAuth>::iterator p = keyring.get_keys().begin();
        p != keyring.get_keys().end();
-       p++) {
+       ++p) {
     KeyServerData::Incremental auth_inc;
     auth_inc.name = p->first;
     auth_inc.auth = p->second; 
@@ -587,7 +653,7 @@ bool AuthMonitor::prepare_command(MMonCommand *m)
   if (!session ||
       (!session->caps.get_allow_all() &&
        !mon->_allowed_command(session, m->cmd))) {
-    mon->reply_command(m, -EACCES, "access denied", rdata, paxos->get_version());
+    mon->reply_command(m, -EACCES, "access denied", rdata, get_version());
     return true;
   }
 
@@ -607,7 +673,8 @@ bool AuthMonitor::prepare_command(MMonCommand *m)
       import_keyring(keyring);
       ss << "imported keyring";
       getline(ss, rs);
-      paxos->wait_for_commit(new Monitor::C_Command(mon, m, 0, rs, paxos->get_version()));
+      wait_for_finished_proposal(new Monitor::C_Command(mon, m, 0, rs, get_version()));
+      //paxos->wait_for_commit(new Monitor::C_Command(mon, m, 0, rs, get_version()));
       return true;
     }
     else if (m->cmd[1] == "add" && m->cmd.size() >= 3) {
@@ -655,7 +722,8 @@ bool AuthMonitor::prepare_command(MMonCommand *m)
 
       ss << "added key for " << auth_inc.name;
       getline(ss, rs);
-      paxos->wait_for_commit(new Monitor::C_Command(mon, m, 0, rs, paxos->get_version()));
+      wait_for_finished_proposal(new Monitor::C_Command(mon, m, 0, rs, get_version()));
+      //paxos->wait_for_commit(new Monitor::C_Command(mon, m, 0, rs, get_version()));
       return true;
     }
     else if ((m->cmd[1] == "get-or-create-key" ||
@@ -705,7 +773,8 @@ bool AuthMonitor::prepare_command(MMonCommand *m)
 	  ::decode(auth_inc, q);
 	  if (auth_inc.op == KeyServerData::AUTH_INC_ADD &&
 	      auth_inc.name == entity) {
-	    paxos->wait_for_commit(new C_RetryMessage(this, m));
+	    wait_for_finished_proposal(new Monitor::C_Command(mon, m, 0, rs, get_version()));
+	    //paxos->wait_for_commit(new C_RetryMessage(this, m));
 	    return true;
 	  }
 	}
@@ -730,7 +799,8 @@ bool AuthMonitor::prepare_command(MMonCommand *m)
       }
 
       getline(ss, rs);
-      paxos->wait_for_commit(new Monitor::C_Command(mon, m, 0, rs, rdata, paxos->get_version()));
+      wait_for_finished_proposal(new Monitor::C_Command(mon, m, 0, rs, rdata, get_version()));
+      //paxos->wait_for_commit(new Monitor::C_Command(mon, m, 0, rs, get_version()));
       return true;
     }
     else if (m->cmd[1] == "caps" && m->cmd.size() >= 3) {
@@ -756,7 +826,8 @@ bool AuthMonitor::prepare_command(MMonCommand *m)
 
       ss << "updated caps for " << auth_inc.name;
       getline(ss, rs);
-      paxos->wait_for_commit(new Monitor::C_Command(mon, m, 0, rs, paxos->get_version()));
+      wait_for_finished_proposal(new Monitor::C_Command(mon, m, 0, rs, get_version()));
+      //paxos->wait_for_commit(new Monitor::C_Command(mon, m, 0, rs, get_version()));
       return true;     
     }
     else if (m->cmd[1] == "del" && m->cmd.size() >= 3) {
@@ -778,7 +849,8 @@ bool AuthMonitor::prepare_command(MMonCommand *m)
 
       ss << "updated";
       getline(ss, rs);
-      paxos->wait_for_commit(new Monitor::C_Command(mon, m, 0, rs, paxos->get_version()));
+      wait_for_finished_proposal(new Monitor::C_Command(mon, m, 0, rs, get_version()));
+      //paxos->wait_for_commit(new Monitor::C_Command(mon, m, 0, rs, get_version()));
       return true;
     }
     else {
@@ -790,7 +862,7 @@ bool AuthMonitor::prepare_command(MMonCommand *m)
 
 done:
   getline(ss, rs, '\0');
-  mon->reply_command(m, err, rs, rdata, paxos->get_version());
+  mon->reply_command(m, err, rs, rdata, get_version());
   return false;
 }
 
