@@ -56,7 +56,7 @@ struct failure_info_t {
   failure_info_t() : num_reports(0) {}
 
   utime_t get_failed_since() {
-    if (max_failed_since == utime_t() && reporters.size()) {
+    if (max_failed_since == utime_t() && !reporters.empty()) {
       // the old max must have canceled; recalculate.
       for (map<int, failure_reporter_t>::iterator p = reporters.begin();
 	   p != reporters.end();
@@ -137,7 +137,8 @@ public:
 private:
   void update_from_paxos();
   void create_pending();  // prepare a new pending
-  void encode_pending(bufferlist &bl);
+  void encode_pending(MonitorDBStore::Transaction *t);
+  virtual void encode_full(MonitorDBStore::Transaction *t);
   void on_active();
 
   void update_msgr_features();
@@ -151,6 +152,9 @@ private:
   bool prepare_update(PaxosServiceMessage *m);
   bool should_propose(double &delay);
 
+  void update_trim();
+  bool should_trim();
+
   bool can_mark_down(int o);
   bool can_mark_up(int o);
   bool can_mark_out(int o);
@@ -213,8 +217,10 @@ private:
 	cmon->_booted(m, logit);
       else if (r == -ECANCELED)
 	m->put();
-      else
+      else if (r == -EAGAIN)
 	cmon->dispatch((PaxosServiceMessage*)m);
+      else
+	assert(0 == "bad C_Booted return value");
     }
   };
 
@@ -224,13 +230,14 @@ private:
     epoch_t e;
     C_ReplyMap(OSDMonitor *o, PaxosServiceMessage *mm, epoch_t ee) : osdmon(o), m(mm), e(ee) {}
     void finish(int r) {
-      if (r >= 0) {
+      if (r >= 0)
 	osdmon->_reply_map(m, e);
-      } else if (r == -ECANCELED) {
+      else if (r == -ECANCELED)
 	m->put();
-      } else {
+      else if (r == -EAGAIN)
 	osdmon->dispatch(m);
-      }
+      else
+	assert(0 == "bad C_ReplyMap return value");
     }    
   };
   struct C_PoolOp : public Context {
@@ -245,13 +252,14 @@ private:
 	reply_data = *rd;
     }
     void finish(int r) {
-      if (r >= 0) {
+      if (r >= 0)
 	osdmon->_pool_op_reply(m, replyCode, epoch, &reply_data);
-      } else if (r == -ECANCELED) {
+      else if (r == -ECANCELED)
 	m->put();
-      } else {
+      else if (r == -EAGAIN)
 	osdmon->dispatch(m);
-      }
+      else
+	assert(0 == "bad C_PoolOp return value");
     }
   };
 
@@ -259,7 +267,9 @@ private:
   bool prepare_remove_snaps(class MRemoveSnaps *m);
 
  public:
-  OSDMonitor(Monitor *mn, Paxos *p);
+  OSDMonitor(Monitor *mn, Paxos *p, string service_name)
+  : PaxosService(mn, p, service_name),
+    thrash_map(0), thrash_last_up_osd(-1) { }
 
   void tick();  // check state, take actions
 
