@@ -17,7 +17,7 @@
 #include "Monitor.h"
 #include "MDSMonitor.h"
 #include "OSDMonitor.h"
-#include "MonitorStore.h"
+#include "MonitorDBStore.h"
 
 #include "messages/MPGStats.h"
 #include "messages/MPGStatsAck.h"
@@ -34,6 +34,9 @@
 #include "common/Formatter.h"
 #include "common/ceph_argparse.h"
 #include "common/perf_counters.h"
+#include "common/TextTable.h"
+
+#include "include/stringify.h"
 
 #include "osd/osd_types.h"
 
@@ -50,13 +53,6 @@ static ostream& _prefix(std::ostream *_dout, const Monitor *mon, const PGMap& pg
 		<< ").pg v" << pg_map.version << " ";
 }
 
-PGMonitor::PGMonitor(Monitor *mn, Paxos *p)
-  : PaxosService(mn, p),
-    need_check_down_pgs(false)
-{ }
-
-PGMonitor::~PGMonitor() {}
-
 /*
  Tick function to update the map based on performance every N seconds
 */
@@ -116,8 +112,7 @@ void PGMonitor::update_logger()
 
 void PGMonitor::tick() 
 {
-  if (!paxos->is_active() ||
-      !mon->is_all_paxos_recovered()) return;
+  if (!is_active()) return;
 
   update_from_paxos();
   handle_osd_timeouts();
@@ -143,34 +138,40 @@ void PGMonitor::create_initial()
 
 void PGMonitor::update_from_paxos()
 {
-  version_t paxosv = paxos->get_version();
-  if (paxosv == pg_map.version)
+  version_t version = get_version();
+  if (version == pg_map.version)
     return;
-  assert(paxosv >= pg_map.version);
+  assert(version >= pg_map.version);
 
-  if (pg_map.version != paxos->get_stashed_version()) {
-    bufferlist latest;
-    version_t v = paxos->get_stashed(latest);
-    dout(7) << "update_from_paxos loading latest full pgmap v" << v << dendl;
+  /* Obtain latest full pgmap version, if available and whose version is
+   * greater than the current pgmap's version.
+   */
+  version_t latest_full = get_version_latest_full();
+  if ((latest_full > 0) && (latest_full > pg_map.version)) {
+    bufferlist latest_bl;
+    int err = get_version_full(latest_full, latest_bl);
+    assert(err == 0);
+    dout(7) << __func__ << " loading latest full pgmap v"
+	    << latest_full << dendl;
     try {
       PGMap tmp_pg_map;
-      bufferlist::iterator p = latest.begin();
+      bufferlist::iterator p = latest_bl.begin();
       tmp_pg_map.decode(p);
       pg_map = tmp_pg_map;
-    }
-    catch (const std::exception &e) {
-      dout(0) << "update_from_paxos: error parsing update: "
+    } catch (const std::exception& e) {
+      dout(0) << __func__ << ": error parsing update: "
 	      << e.what() << dendl;
       assert(0 == "update_from_paxos: error parsing update");
       return;
     }
-  } 
+  }
 
   // walk through incrementals
-  while (paxosv > pg_map.version) {
+  while (version > pg_map.version) {
     bufferlist bl;
-    bool success = paxos->read(pg_map.version+1, bl);
-    assert(success);
+    int err = get_version(pg_map.version+1, bl);
+    assert(err == 0);
+    assert(bl.length());
 
     dout(7) << "update_from_paxos  applying incremental " << pg_map.version+1 << dendl;
     PGMap::Incremental inc;
@@ -193,26 +194,23 @@ void PGMonitor::update_from_paxos()
       last_sent_pg_create.clear();  // reset pg_create throttle timer
   }
 
-  assert(paxosv == pg_map.version);
-
-  // save latest
-  bufferlist bl;
-  pg_map.encode(bl, mon->get_quorum_features());
-  paxos->stash_latest(paxosv, bl);
+  assert(version == pg_map.version);
 
+  /* If we dump the summaries onto the k/v store, they hardly would be useful
+   * without a tool created with reading them in mind.
+   * Comment this out until we decide what is the best course of action.
+   *
   // dump pgmap summaries?  (useful for debugging)
   if (0) {
     stringstream ds;
     pg_map.dump(ds);
     bufferlist d;
     d.append(ds);
-    mon->store->put_bl_sn(d, "pgmap_dump", paxosv);
+    mon->store->put_bl_sn(d, "pgmap_dump", version);
   }
+  */
 
-  unsigned max = g_conf->mon_max_pgmap_epochs;
-  if (mon->is_leader() &&
-      paxosv > max)
-    paxos->trim_to(paxosv - max);
+  update_trim();
 
   send_pg_creates();
 
@@ -230,7 +228,7 @@ void PGMonitor::handle_osd_timeouts()
     return;
   }
 
-  if (mon->osdmon()->paxos->is_writeable())
+  if (mon->osdmon()->is_writeable())
     mon->osdmon()->handle_osd_timeouts(now, last_osd_report);
 }
 
@@ -253,29 +251,56 @@ void PGMonitor::create_pending()
   dout(10) << "create_pending v " << pending_inc.version << dendl;
 }
 
-void PGMonitor::encode_pending(bufferlist &bl)
+void PGMonitor::encode_pending(MonitorDBStore::Transaction *t)
 {
-  dout(10) << "encode_pending v " << pending_inc.version << dendl;
-  assert(paxos->get_version() + 1 == pending_inc.version);
+  version_t version = pending_inc.version;
+  dout(10) << __func__ << " v " << version << dendl;
+  assert(get_version() + 1 == version);
   pending_inc.stamp = ceph_clock_now(g_ceph_context);
+
+  bufferlist bl;
   pending_inc.encode(bl, mon->get_quorum_features());
+
+  put_version(t, version, bl);
+  put_last_committed(t, version);
 }
 
+void PGMonitor::encode_full(MonitorDBStore::Transaction *t)
+{
+  dout(10) << __func__ << " pgmap v " << pg_map.version << dendl;
+  assert(get_version() == pg_map.version);
+
+  bufferlist full_bl;
+  pg_map.encode(full_bl, mon->get_quorum_features());
+
+  put_version_full(t, pg_map.version, full_bl);
+  put_version_latest_full(t, pg_map.version);
+}
+
+void PGMonitor::update_trim()
+{
+  unsigned max = g_conf->mon_max_pgmap_epochs;
+  version_t version = get_version();
+  if (mon->is_leader() && (version > max))
+    set_trim_to(version - max);
+}
+
+
 bool PGMonitor::preprocess_query(PaxosServiceMessage *m)
 {
   dout(10) << "preprocess_query " << *m << " from " << m->get_orig_source_inst() << dendl;
   switch (m->get_type()) {
   case CEPH_MSG_STATFS:
-    handle_statfs((MStatfs*)m);
+    handle_statfs(static_cast<MStatfs*>(m));
     return true;
   case MSG_GETPOOLSTATS:
-    return preprocess_getpoolstats((MGetPoolStats*)m);
+    return preprocess_getpoolstats(static_cast<MGetPoolStats*>(m));
     
   case MSG_PGSTATS:
-    return preprocess_pg_stats((MPGStats*)m);
+    return preprocess_pg_stats(static_cast<MPGStats*>(m));
 
   case MSG_MON_COMMAND:
-    return preprocess_command((MMonCommand*)m);
+    return preprocess_command(static_cast<MMonCommand*>(m));
 
 
   default:
@@ -293,7 +318,7 @@ bool PGMonitor::prepare_update(PaxosServiceMessage *m)
     return prepare_pg_stats((MPGStats*)m);
 
   case MSG_MON_COMMAND:
-    return prepare_command((MMonCommand*)m);
+    return prepare_command(static_cast<MMonCommand*>(m));
 
   default:
     assert(0);
@@ -323,7 +348,7 @@ void PGMonitor::handle_statfs(MStatfs *statfs)
   }
 
   // fill out stfs
-  reply = new MStatfsReply(mon->monmap->fsid, statfs->get_tid(), paxos->get_version());
+  reply = new MStatfsReply(mon->monmap->fsid, statfs->get_tid(), get_version());
 
   // these are in KB.
   reply->h.st.kb = pg_map.osd_sum.kb;
@@ -355,11 +380,11 @@ bool PGMonitor::preprocess_getpoolstats(MGetPoolStats *m)
     goto out;
   }
   
-  reply = new MGetPoolStatsReply(m->fsid, m->get_tid(), paxos->get_version());
+  reply = new MGetPoolStatsReply(m->fsid, m->get_tid(), get_version());
 
   for (list<string>::iterator p = m->pools.begin();
        p != m->pools.end();
-       p++) {
+       ++p) {
     int64_t poolid = mon->osdmon()->osdmap.lookup_pg_pool_name(p->c_str());
     if (poolid < 0)
       continue;
@@ -395,7 +420,7 @@ bool PGMonitor::preprocess_pg_stats(MPGStats *stats)
   // First, just see if they need a new osdmap. But
   // only if they've had the map for a while.
   if (stats->had_map_for > 30.0 && 
-      mon->osdmon()->paxos->is_readable() &&
+      mon->osdmon()->is_readable() &&
       stats->epoch < mon->osdmon()->osdmap.get_epoch())
     mon->osdmon()->send_latest_now_nodelete(stats, stats->epoch+1);
 
@@ -486,7 +511,7 @@ bool PGMonitor::prepare_pg_stats(MPGStats *stats)
   ack->set_tid(stats->get_tid());
   for (map<pg_t,pg_stat_t>::iterator p = stats->pg_stat.begin();
        p != stats->pg_stat.end();
-       p++) {
+       ++p) {
     pg_t pgid = p->first;
     ack->pg_stat[pgid] = p->second.reported;
 
@@ -525,7 +550,7 @@ bool PGMonitor::prepare_pg_stats(MPGStats *stats)
     */
   }
   
-  paxos->wait_for_commit(new C_Stats(this, stats, ack));
+  wait_for_finished_proposal(new C_Stats(this, stats, ack));
   return true;
 }
 
@@ -561,15 +586,15 @@ void PGMonitor::check_osd_map(epoch_t epoch)
     return;
   }
 
-  if (!mon->osdmon()->paxos->is_readable()) {
+  if (!mon->osdmon()->is_readable()) {
     dout(10) << "check_osd_map -- osdmap not readable, waiting" << dendl;
-    mon->osdmon()->paxos->wait_for_readable(new RetryCheckOSDMap(this, epoch));
+    mon->osdmon()->wait_for_readable(new RetryCheckOSDMap(this, epoch));
     return;
   }
 
-  if (!paxos->is_writeable()) {
+  if (!is_writeable()) {
     dout(10) << "check_osd_map -- pgmap not writeable, waiting" << dendl;
-    paxos->wait_for_writeable(new RetryCheckOSDMap(this, epoch));
+    wait_for_writeable(new RetryCheckOSDMap(this, epoch));
     return;
   }
 
@@ -579,12 +604,14 @@ void PGMonitor::check_osd_map(epoch_t epoch)
        e++) {
     dout(10) << "check_osd_map applying osdmap e" << e << " to pg_map" << dendl;
     bufferlist bl;
-    mon->store->get_bl_sn_safe(bl, "osdmap", e);
+    int err = mon->osdmon()->get_version(e, bl);
+    assert(err == 0);
+
     assert(bl.length());
     OSDMap::Incremental inc(bl);
     for (map<int32_t,uint32_t>::iterator p = inc.new_weight.begin();
 	 p != inc.new_weight.end();
-	 p++)
+	 ++p)
       if (p->second == CEPH_OSD_OUT) {
 	dout(10) << "check_osd_map  osd." << p->first << " went OUT" << dendl;
 	pending_inc.osd_stat_rm.insert(p->first);
@@ -689,7 +716,7 @@ bool PGMonitor::register_new_pgs()
   int created = 0;
   for (map<int64_t,pg_pool_t>::iterator p = osdmap->pools.begin();
        p != osdmap->pools.end();
-       p++) {
+       ++p) {
     int64_t poolid = p->first;
     pg_pool_t &pool = p->second;
     int ruleno = pool.get_crush_ruleset();
@@ -720,7 +747,7 @@ bool PGMonitor::register_new_pgs()
   int removed = 0;
   for (set<pg_t>::iterator p = pg_map.creating_pgs.begin();
        p != pg_map.creating_pgs.end();
-       p++) {
+       ++p) {
     if (p->preferred() >= 0) {
       dout(20) << " removing creating_pg " << *p << " because it is localized and obsolete" << dendl;
       pending_inc.pg_remove.insert(*p);
@@ -766,7 +793,7 @@ void PGMonitor::send_pg_creates()
   
   for (set<pg_t>::iterator p = pg_map.creating_pgs.begin();
        p != pg_map.creating_pgs.end();
-       p++) {
+       ++p) {
     pg_t pgid = *p;
     pg_t on = pgid;
     pg_stat_t& s = pg_map.pg_stat[pgid];
@@ -870,6 +897,157 @@ bool PGMonitor::check_down_pgs()
   return ret;
 }
 
+inline string percentify(const float& a) {
+  stringstream ss;
+  if (a < 0.01)
+    ss << "0";
+  else
+    ss << std::fixed << std::setprecision(2) << a;
+  return ss.str();
+}
+
+//void PGMonitor::dump_object_stat_sum(stringstream& ss, Formatter *f,
+void PGMonitor::dump_object_stat_sum(TextTable &tbl, Formatter *f,
+    object_stat_sum_t &sum, bool verbose)
+{
+  if (f) {
+    f->dump_int("kb_used", SHIFT_ROUND_UP(sum.num_bytes, 10));
+    f->dump_int("bytes_used", sum.num_bytes);
+    f->dump_int("objects", sum.num_objects);
+    if (verbose) {
+      f->dump_int("rd", sum.num_rd);
+      f->dump_int("rd_kb", sum.num_rd_kb);
+      f->dump_int("wr", sum.num_wr);
+      f->dump_int("wr_kb", sum.num_wr_kb);
+    }
+  } else {
+    tbl << stringify(si_t(sum.num_bytes));
+    int64_t kb_used = SHIFT_ROUND_UP(sum.num_bytes, 10);
+    tbl << percentify(((float)kb_used / pg_map.osd_sum.kb)*100);
+    tbl << sum.num_objects;
+    if (verbose) {
+      tbl << stringify(si_t(sum.num_rd))
+          << stringify(si_t(sum.num_wr));
+    }
+  }
+}
+
+void PGMonitor::dump_pool_stats(stringstream &ss, Formatter *f, bool verbose)
+{
+  TextTable tbl;
+
+  if (f) {
+    f->open_array_section("pools");
+  } else {
+    tbl.define_column("NAME", TextTable::LEFT, TextTable::LEFT);
+    tbl.define_column("ID", TextTable::LEFT, TextTable::LEFT);
+    if (verbose)
+      tbl.define_column("CATEGORY", TextTable::LEFT, TextTable::LEFT);
+    tbl.define_column("USED", TextTable::LEFT, TextTable::LEFT);
+    tbl.define_column("\%USED", TextTable::LEFT, TextTable::LEFT);
+    tbl.define_column("OBJECTS", TextTable::LEFT, TextTable::LEFT);
+    if (verbose) {
+      tbl.define_column("READ", TextTable::LEFT, TextTable::LEFT);
+      tbl.define_column("WRITE", TextTable::LEFT, TextTable::LEFT);
+    }
+  }
+
+  OSDMap &osdmap = mon->osdmon()->osdmap;
+  for (map<int64_t,pg_pool_t>::const_iterator p = osdmap.get_pools().begin();
+       p != osdmap.get_pools().end(); ++p) {
+    int64_t pool_id = p->first;
+    if ((pool_id < 0) || (pg_map.pg_pool_sum.count(pool_id) == 0))
+      continue;
+    string pool_name = osdmap.get_pool_name(pool_id);
+    pool_stat_t &stat = pg_map.pg_pool_sum[pool_id];
+
+    if (f) {
+      f->open_object_section("pool");
+      f->dump_string("name", pool_name);
+      f->dump_int("id", pool_id);
+      f->open_object_section("stats");
+    } else {
+      tbl << pool_name
+          << pool_id;
+      if (verbose)
+        tbl << "-";
+    }
+    dump_object_stat_sum(tbl, f, stat.stats.sum, verbose);
+    if (f)
+      f->close_section(); // stats
+    else
+      tbl << TextTable::endrow;
+
+    if (verbose) {
+      if (f)
+        f->open_array_section("categories");
+
+      for (map<string,object_stat_sum_t>::iterator it = stat.stats.cat_sum.begin();
+          it != stat.stats.cat_sum.end(); ++it) {
+        if (f) {
+          f->open_object_section(it->first.c_str());
+        } else {
+          tbl << ""
+              << ""
+              << it->first;
+        }
+        dump_object_stat_sum(tbl, f, it->second, verbose);
+        if (f)
+          f->close_section(); // category name
+        else
+          tbl << TextTable::endrow;
+      }
+      if (f)
+        f->close_section(); // categories
+    }
+    if (f)
+      f->close_section(); // pool
+  }
+  if (f)
+    f->close_section();
+  else {
+    ss << "POOLS:\n";
+    tbl.set_indent(4);
+    ss << tbl;
+  }
+}
+
+void PGMonitor::dump_fs_stats(stringstream &ss, Formatter *f, bool verbose)
+{
+  if (f) {
+    f->open_object_section("stats");
+    f->dump_int("total_space", pg_map.osd_sum.kb);
+    f->dump_int("total_used", pg_map.osd_sum.kb_used);
+    f->dump_int("total_avail", pg_map.osd_sum.kb_avail);
+    if (verbose) {
+      f->dump_int("total_objects", pg_map.pg_sum.stats.sum.num_objects);
+    }
+    f->close_section();
+  } else {
+    TextTable tbl;
+    tbl.define_column("SIZE", TextTable::LEFT, TextTable::LEFT);
+    tbl.define_column("AVAIL", TextTable::LEFT, TextTable::LEFT);
+    tbl.define_column("RAW USED", TextTable::LEFT, TextTable::LEFT);
+    tbl.define_column("\%RAW USED", TextTable::LEFT, TextTable::LEFT);
+    if (verbose) {
+      tbl.define_column("OBJECTS", TextTable::LEFT, TextTable::LEFT);
+    }
+    tbl << stringify(si_t(pg_map.osd_sum.kb))
+        << stringify(si_t(pg_map.osd_sum.kb_avail))
+        << stringify(si_t(pg_map.osd_sum.kb_used));
+    tbl << percentify(((float)pg_map.osd_sum.kb_used / pg_map.osd_sum.kb)*100);
+    if (verbose) {
+      tbl << stringify(si_t(pg_map.pg_sum.stats.sum.num_objects));
+    }
+    tbl << TextTable::endrow;
+
+    ss << "GLOBAL:\n";
+    tbl.set_indent(4);
+    ss << tbl;
+  }
+}
+
+
 void PGMonitor::dump_info(Formatter *f)
 {
   f->open_object_section("pgmap");
@@ -888,7 +1066,7 @@ bool PGMonitor::preprocess_command(MMonCommand *m)
       (!session->caps.get_allow_all() &&
        !session->caps.check_privileges(PAXOS_PGMAP, MON_CAP_R) &&
        !mon->_allowed_command(session, m->cmd))) {
-    mon->reply_command(m, -EACCES, "access denied", rdata, paxos->get_version());
+    mon->reply_command(m, -EACCES, "access denied", rdata, get_version());
     return true;
   }
 
@@ -1083,7 +1261,7 @@ bool PGMonitor::preprocess_command(MMonCommand *m)
   if (r != -1) {
     string rs;
     getline(ss, rs);
-    mon->reply_command(m, r, rs, rdata, paxos->get_version());
+    mon->reply_command(m, r, rs, rdata, get_version());
     return true;
   } else
     return false;
@@ -1103,7 +1281,7 @@ bool PGMonitor::prepare_command(MMonCommand *m)
       (!session->caps.get_allow_all() &&
        !session->caps.check_privileges(PAXOS_PGMAP, MON_CAP_W) &&
        !mon->_allowed_command(session, m->cmd))) {
-    mon->reply_command(m, -EACCES, "access denied", paxos->get_version());
+    mon->reply_command(m, -EACCES, "access denied", get_version());
     return true;
   }
 
@@ -1132,7 +1310,7 @@ bool PGMonitor::prepare_command(MMonCommand *m)
     }
     ss << "pg " << m->cmd[2] << " now creating, ok";
     getline(ss, rs);
-    paxos->wait_for_commit(new Monitor::C_Command(mon, m, 0, rs, paxos->get_version()));
+    wait_for_finished_proposal(new Monitor::C_Command(mon, m, 0, rs, get_version()));
     return true;
   }
   else if (m->cmd.size() > 1 && m->cmd[1] == "set_full_ratio") {
@@ -1148,7 +1326,7 @@ bool PGMonitor::prepare_command(MMonCommand *m)
       goto out;
     }
     pending_inc.full_ratio = n;
-    paxos->wait_for_commit(new Monitor::C_Command(mon, m, 0, rs, paxos->get_version()));
+    wait_for_finished_proposal(new Monitor::C_Command(mon, m, 0, rs, get_version()));
     return true;
   }
   else if (m->cmd.size() > 1 && m->cmd[1] == "set_nearfull_ratio") {
@@ -1164,7 +1342,7 @@ bool PGMonitor::prepare_command(MMonCommand *m)
       goto out;
     }
     pending_inc.nearfull_ratio = n;
-    paxos->wait_for_commit(new Monitor::C_Command(mon, m, 0, rs, paxos->get_version()));
+    wait_for_finished_proposal(new Monitor::C_Command(mon, m, 0, rs, get_version()));
     return true;
   }
 
@@ -1172,7 +1350,7 @@ bool PGMonitor::prepare_command(MMonCommand *m)
   getline(ss, rs);
   if (r < 0 && rs.length() == 0)
     rs = cpp_strerror(r);
-  mon->reply_command(m, r, rs, paxos->get_version());
+  mon->reply_command(m, r, rs, get_version());
   return false;
 }
 
@@ -1278,7 +1456,7 @@ void PGMonitor::get_health(list<pair<health_status_t,string> >& summary,
   }
 
   if (!note.empty()) {
-    for (map<string,int>::iterator p = note.begin(); p != note.end(); p++) {
+    for (map<string,int>::iterator p = note.begin(); p != note.end(); ++p) {
       ostringstream ss;
       ss << p->second << " pgs " << p->first;
       summary.push_back(make_pair(HEALTH_WARN, ss.str()));
@@ -1287,7 +1465,7 @@ void PGMonitor::get_health(list<pair<health_status_t,string> >& summary,
       for (hash_map<pg_t,pg_stat_t>::const_iterator p = pg_map.pg_stat.begin();
 	   p != pg_map.pg_stat.end();
 	   ++p) {
-	if (p->second.state & (PG_STATE_STALE |
+	if ((p->second.state & (PG_STATE_STALE |
 			       PG_STATE_DOWN |
 			       PG_STATE_DEGRADED |
 			       PG_STATE_INCONSISTENT |
@@ -1299,7 +1477,7 @@ void PGMonitor::get_health(list<pair<health_status_t,string> >& summary,
 			       PG_STATE_INCOMPLETE |
 			       PG_STATE_BACKFILL_WAIT |
 			       PG_STATE_BACKFILL |
-			       PG_STATE_BACKFILL_TOOFULL) &&
+			       PG_STATE_BACKFILL_TOOFULL)) &&
 	    stuck_pgs.count(p->first) == 0) {
 	  ostringstream ss;
 	  ss << "pg " << p->first << " is " << pg_state_string(p->second.state);
