@@ -39,6 +39,7 @@ using namespace std;
 #include "messages/MMonProbe.h"
 #include "messages/MMonJoin.h"
 #include "messages/MMonElection.h"
+#include "messages/MMonSync.h"
 
 #include "messages/MLog.h"
 #include "messages/MLogAck.h"
@@ -83,6 +84,7 @@ using namespace std;
 #include "messages/MMonGetMap.h"
 #include "messages/MMonGetVersion.h"
 #include "messages/MMonGetVersionReply.h"
+#include "messages/MMonHealth.h"
 
 #include "messages/MAuth.h"
 #include "messages/MAuthReply.h"
@@ -310,6 +312,9 @@ Message *decode_message(CephContext *cct, ceph_msg_header& header, ceph_msg_foot
   case MSG_MON_ELECTION:
     m = new MMonElection;
     break;
+  case MSG_MON_SYNC:
+    m = new MMonSync;
+    break;
 
   case MSG_LOG:
     m = new MLog;
@@ -608,6 +613,10 @@ Message *decode_message(CephContext *cct, ceph_msg_header& header, ceph_msg_foot
     m = new MTimeCheck();
     break;
 
+  case MSG_MON_HEALTH:
+    m = new MMonHealth();
+    break;
+
     // -- simple messages without payload --
 
   case CEPH_MSG_SHUTDOWN:
@@ -654,6 +663,9 @@ Message *decode_message(CephContext *cct, ceph_msg_header& header, ceph_msg_foot
       lderr(cct) << "failed to decode message of type " << type
 		 << " v" << header.version
 		 << ": " << e.what() << dendl;
+      ldout(cct, 30) << "dump: \n";
+      m->get_payload().hexdump(*_dout);
+      *_dout << dendl;
       if (cct->_conf->ms_die_on_bad_msg)
 	assert(0);
     }
