@@ -212,6 +212,8 @@ int Pipe::accept()
 {
   ldout(msgr->cct,10) << "accept" << dendl;
 
+  set_socket_options();
+
   // my creater gave me sd via accept()
   assert(state == STATE_ACCEPTING);
   
@@ -586,7 +588,7 @@ int Pipe::accept()
     ldout(msgr->cct,10) << "accept re-queuing on out_seq " << out_seq << " in_seq " << in_seq << dendl;
     for (map<int, list<Message*> >::iterator p = existing->out_q.begin();
          p != existing->out_q.end();
-         p++)
+         ++p)
       out_q[p->first].splice(out_q[p->first].begin(), p->second);
   }
   existing->pipe_lock.Unlock();
@@ -709,6 +711,27 @@ int Pipe::accept()
   return -1;
 }
 
+void Pipe::set_socket_options()
+{
+  // disable Nagle algorithm?
+  if (msgr->cct->_conf->ms_tcp_nodelay) {
+    int flag = 1;
+    int r = ::setsockopt(sd, IPPROTO_TCP, TCP_NODELAY, (char*)&flag, sizeof(flag));
+    if (r < 0) {
+      r = -errno;
+      ldout(msgr->cct,0) << "couldn't set TCP_NODELAY: " << cpp_strerror(r) << dendl;
+    }
+  }
+  if (msgr->cct->_conf->ms_tcp_rcvbuf) {
+    int size = msgr->cct->_conf->ms_tcp_rcvbuf;
+    int r = ::setsockopt(sd, SOL_SOCKET, SO_RCVBUF, (void*)&size, sizeof(size));
+    if (r < 0)  {
+      r = -errno;
+      ldout(msgr->cct,0) << "couldn't set SO_RCVBUF to " << size << ": " << cpp_strerror(r) << dendl;
+    }
+  }
+}
+
 int Pipe::connect()
 {
   bool got_bad_auth = false;
@@ -758,13 +781,7 @@ int Pipe::connect()
     goto fail;
   }
 
-  // disable Nagle algorithm?
-  if (conf->ms_tcp_nodelay) {
-    int flag = 1;
-    int r = ::setsockopt(sd, IPPROTO_TCP, TCP_NODELAY, (char*)&flag, sizeof(flag));
-    if (r < 0) 
-      ldout(msgr->cct,0) << "connect couldn't set TCP_NODELAY: " << strerror_r(errno, buf, sizeof(buf)) << dendl;
-  }
+  set_socket_options();
 
   // verify banner
   // FIXME: this should be non-blocking, or in some other way verify the banner as we get it.
@@ -1110,13 +1127,13 @@ void Pipe::discard_out_queue()
 {
   ldout(msgr->cct,10) << "discard_queue" << dendl;
 
-  for (list<Message*>::iterator p = sent.begin(); p != sent.end(); p++) {
+  for (list<Message*>::iterator p = sent.begin(); p != sent.end(); ++p) {
     ldout(msgr->cct,20) << "  discard " << *p << dendl;
     (*p)->put();
   }
   sent.clear();
-  for (map<int,list<Message*> >::iterator p = out_q.begin(); p != out_q.end(); p++)
-    for (list<Message*>::iterator r = p->second.begin(); r != p->second.end(); r++) {
+  for (map<int,list<Message*> >::iterator p = out_q.begin(); p != out_q.end(); ++p)
+    for (list<Message*>::iterator r = p->second.begin(); r != p->second.end(); ++r) {
       ldout(msgr->cct,20) << "  discard " << *r << dendl;
       (*r)->put();
     }
@@ -1568,9 +1585,9 @@ static void alloc_aligned_buffer(bufferlist& data, unsigned len, unsigned off)
 {
   // create a buffer to read into that matches the data alignment
   unsigned left = len;
-  unsigned head = 0;
   if (off & ~CEPH_PAGE_MASK) {
     // head
+    unsigned head = 0;
     head = MIN(CEPH_PAGE_SIZE - (off & ~CEPH_PAGE_MASK), left);
     bufferptr bp = buffer::create(head);
     data.push_back(bp);
@@ -1633,10 +1650,10 @@ int Pipe::read_message(Message **pm)
   int aborted;
   Message *message;
   utime_t recv_stamp = ceph_clock_now(msgr->cct);
-  bool waited_on_throttle = false;
 
   uint64_t message_size = header.front_len + header.middle_len + header.data_len;
   if (message_size) {
+    bool waited_on_throttle = false;
     if (policy.throttler) {
       ldout(msgr->cct,10) << "reader wants " << message_size << " from policy throttler "
 	       << policy.throttler->get_current() << "/"
@@ -1981,7 +1998,7 @@ int Pipe::write_message(ceph_msg_header& header, ceph_msg_footer& footer, buffer
     if (left == 0)
       break;
     while (b_off == (int)pb->length()) {
-      pb++;
+      ++pb;
       b_off = 0;
     }
   }
