@@ -250,7 +250,7 @@ void JournalingObjectStore::_op_journal_transactions(
     unsigned data_len = 0;
     int data_align = -1; // -1 indicates that we don't care about the alignment
     for (list<ObjectStore::Transaction*>::iterator p = tls.begin();
-	 p != tls.end(); p++) {
+	 p != tls.end(); ++p) {
       ObjectStore::Transaction *t = *p;
       if (t->get_data_length() > data_len &&
 	(int)t->get_data_length() >= g_conf->journal_align_min_size) {
