@@ -48,6 +48,7 @@ using namespace std;
 #include <ext/hash_set>
 using namespace __gnu_cxx;
 
+#include "Watch.h"
 #include "common/shared_cache.hpp"
 #include "common/simple_cache.hpp"
 #include "common/sharedptr_registry.hpp"
@@ -149,7 +150,6 @@ public:
     on_deletion_complete.push_front(completion);
   }
   ~DeletingState() {
-    Mutex::Locker l(lock);
     for (list<Context *>::iterator i = on_deletion_complete.begin();
 	 i != on_deletion_complete.end();
 	 ++i) {
@@ -295,7 +295,11 @@ public:
   // -- Watch --
   Mutex watch_lock;
   SafeTimer watch_timer;
-  Watch *watch;
+  uint64_t next_notif_id;
+  uint64_t get_next_id(epoch_t cur_epoch) {
+    Mutex::Locker l(watch_lock);
+    return (((uint64_t)cur_epoch) << 32) | ((uint64_t)(next_notif_id++));
+  }
 
   // -- Backfill Request Scheduling --
   Mutex backfill_request_lock;
@@ -393,6 +397,15 @@ public:
   void expand_pg_num(OSDMapRef old_map,
 		     OSDMapRef new_map);
 
+  // -- OSD Full Status --
+  Mutex full_status_lock;
+  enum s_names { NONE, NEAR, FULL } cur_state;
+  time_t last_msg;
+  float get_full_ratio();
+  float get_nearfull_ratio();
+  void check_nearfull_warning(const osd_stat_t &stat);
+  bool check_failsafe_full();
+
   OSDService(OSD *osd);
 };
 class OSD : public Dispatcher,
@@ -461,6 +474,13 @@ public:
     return hobject_t(sobject_t(object_t(foo), 0)); 
   }
 
+  static hobject_t make_snapmapper_oid() {
+    return hobject_t(
+      sobject_t(
+	object_t("snapmapper"),
+	0));
+  }
+
   static hobject_t make_pg_log_oid(pg_t pg) {
     stringstream ss;
     ss << "pglog_" << pg;
@@ -526,7 +546,7 @@ public:
     int64_t auid;
     epoch_t last_sent_epoch;
     Connection *con;
-    std::map<void *, pg_t> watches;
+    WatchConState wstate;
 
     Session() : auid(-1), last_sent_epoch(0), con(0) {}
   };
@@ -896,7 +916,7 @@ protected:
   void wake_all_pg_waiters() {
     for (map<pg_t, list<OpRequestRef> >::iterator p = waiting_for_pg.begin();
 	 p != waiting_for_pg.end();
-	 p++)
+	 ++p)
       take_waiters_front(p->second);
     waiting_for_pg.clear();
   }
@@ -1468,17 +1488,6 @@ public:
 
   int init_op_flags(OpRequestRef op);
 
-
-  void put_object_context(void *_obc, pg_t pgid);
-  void complete_notify(void *notif, void *obc);
-  void ack_notification(entity_name_t& peer_addr, void *notif, void *obc,
-			ReplicatedPG *pg);
-  void handle_notify_timeout(void *notif);
-  void disconnect_session_watches(Session *session);
-  void handle_watch_timeout(void *obc,
-			    ReplicatedPG *pg,
-			    entity_name_t entity,
-			    utime_t expire);
   OSDService service;
   friend class OSDService;
 };
