@@ -227,6 +227,8 @@ bool pg_t::is_split(unsigned old_pg_num, unsigned new_pg_num, set<pg_t> *childre
 }
 
 unsigned pg_t::get_split_bits(unsigned pg_num) const {
+  if (pg_num == 1)
+    return 0;
   assert(pg_num > 1);
 
   // Find unique p such that pg_num \in [2^(p-1), 2^p)
@@ -615,7 +617,7 @@ snapid_t pg_pool_t::snap_exists(const char *s) const
 {
   for (map<snapid_t,pool_snap_info_t>::const_iterator p = snaps.begin();
        p != snaps.end();
-       p++)
+       ++p)
     if (p->second.name == s)
       return p->second.snapid;
   return 0;
@@ -662,7 +664,7 @@ SnapContext pg_pool_t::get_snap_context() const
   unsigned i = 0;
   for (map<snapid_t, pool_snap_info_t>::const_reverse_iterator p = snaps.rbegin();
        p != snaps.rend();
-       p++)
+       ++p)
     s[i++] = p->first;
   return SnapContext(get_snap_seq(), s);
 }
@@ -1618,6 +1620,13 @@ bool pg_interval_t::check_new_interval(
     if (!i.acting.empty() &&
 	i.acting.size() >=
 	osdmap->get_pools().find(pool_id)->second.min_size) {
+      if (out)
+	*out << "generate_past_intervals " << i
+	     << ": not rw,"
+	     << " up_thru " << lastmap->get_up_thru(i.acting[0])
+	     << " up_from " << lastmap->get_up_from(i.acting[0])
+	     << " last_epoch_clean " << last_epoch_clean
+	     << std::endl;
       if (lastmap->get_up_thru(i.acting[0]) >= i.first &&
 	  lastmap->get_up_from(i.acting[0]) <= i.first) {
 	i.maybe_went_rw = true;
@@ -1944,7 +1953,7 @@ void pg_log_t::copy_after(const pg_log_t &other, eversion_t v)
   tail = other.tail;
   for (list<pg_log_entry_t>::const_reverse_iterator i = other.log.rbegin();
        i != other.log.rend();
-       i++) {
+       ++i) {
     assert(i->version > other.tail);
     if (i->version <= v) {
       // make tail accurate.
@@ -1995,7 +2004,7 @@ ostream& pg_log_t::print(ostream& out) const
   out << *this << std::endl;
   for (list<pg_log_entry_t>::const_iterator p = log.begin();
        p != log.end();
-       p++) 
+       ++p) 
     out << *p << std::endl;
   return out;
 }
@@ -2800,7 +2809,7 @@ void ScrubMap::merge_incr(const ScrubMap &l)
 
   for (map<hobject_t,object>::const_iterator p = l.objects.begin();
        p != l.objects.end();
-       p++){
+       ++p){
     if (p->second.negative) {
       map<hobject_t,object>::iterator q = objects.find(p->first);
       if (q != objects.end()) {
