@@ -80,6 +80,12 @@ class rados_pool_stat_t(Structure):
                 ("num_wr", c_uint64),
                 ("num_wr_kb", c_uint64)]
 
+class rados_cluster_stat_t(Structure):
+    _fields_ = [("kb", c_uint64),
+                ("kb_used", c_uint64),
+                ("kb_avail", c_uint64),
+                ("num_objects", c_uint64)]
+
 class Version(object):
     def __init__(self, major, minor, extra):
         self.major = major
@@ -185,6 +191,17 @@ Rados object in state %s." % (self.state))
             raise make_ex(ret, "error calling connect")
         self.state = "connected"
 
+    def get_cluster_stats(self):
+        stats = rados_cluster_stat_t()
+        ret = self.librados.rados_cluster_stat(self.cluster, byref(stats))
+        if ret < 0:
+            raise make_ex(
+                ret, "Rados.get_cluster_stats(%s): get_stats failed" % self.name)
+        return {'kb': stats.kb,
+                'kb_used': stats.kb_used,
+                'kb_avail': stats.kb_avail,
+                'num_objects': stats.num_objects}
+
     # Returns true if the pool exists; false otherwise.
     def pool_exists(self, pool_name):
         self.require_state("connected")
@@ -242,6 +259,17 @@ Rados object in state %s." % (self.state))
                 break
         return filter(lambda name: name != '', c_names.raw.split('\0'))
 
+    def get_fsid(self):
+        self.require_state("connected")
+        fsid_len = 36
+        fsid = create_string_buffer(fsid_len + 1)
+        ret = self.librados.rados_cluster_fsid(self.cluster,
+                                               byref(fsid),
+                                               fsid_len + 1)
+        if ret < 0:
+            raise make_ex(ret, "error getting cluster fsid")
+        return fsid.value
+
     def open_ioctx(self, ioctx_name):
         self.require_state("connected")
         if not isinstance(ioctx_name, str):
