@@ -27,6 +27,7 @@
 #include <map>
 #include "include/types.h"
 #include "include/utime.h"
+#include "rgw_acl.h"
 
 using namespace std;
 
@@ -245,6 +246,8 @@ public:
   int get_int(const char *name, int def_val = 0);
   bool get_bool(const char *name, bool def_val = 0);
   size_t get_size(const char *name, size_t def_val = 0);
+  bool exists(const char *name);
+  bool exists_prefix(const char *prefix);
 };
 
 class RGWConf {
@@ -509,7 +512,7 @@ struct rgw_bucket {
 };
 WRITE_CLASS_ENCODER(rgw_bucket)
 
-inline ostream& operator<<(ostream& out, const rgw_bucket b) {
+inline ostream& operator<<(ostream& out, const rgw_bucket &b) {
   out << b.name;
   if (b.name.compare(b.pool))
     out << "(@" << b.pool << "[" << b.marker << "])";
@@ -597,7 +600,8 @@ struct req_state {
    rgw_bucket bucket;
    string bucket_name_str;
    string object_str;
-   string bucket_owner;
+   ACLOwner bucket_owner;
+   ACLOwner owner;
 
    map<string, string> x_meta_map;
    bool has_bad_meta;
@@ -607,6 +611,7 @@ struct req_state {
    RGWAccessControlPolicy *object_acl;
 
    string canned_acl;
+   bool has_acl_header;
    const char *copy_source;
    const char *http_auth;
 
@@ -913,7 +918,7 @@ public:
 };
 WRITE_CLASS_ENCODER(rgw_obj)
 
-inline ostream& operator<<(ostream& out, const rgw_obj o) {
+inline ostream& operator<<(ostream& out, const rgw_obj &o) {
   return out << o.bucket.name << ":" << o.object;
 }
 
@@ -997,6 +1002,9 @@ static inline const char *rgw_obj_category_name(RGWObjCategory category)
 }
 
 extern string rgw_string_unquote(const string& s);
+extern void parse_csv_string(const string& ival, vector<string>& ovals);
+extern int parse_key_value(string& in_str, string& key, string& val);
+extern int parse_key_value(string& in_str, const char *delim, string& key, string& val);
 /** time parsing */
 extern int parse_time(const char *time_str, time_t *time);
 extern bool parse_rfc2616(const char *s, struct tm *t);
