@@ -210,7 +210,6 @@ static int get_policy_from_attr(CephContext *cct, RGWRados *store, void *ctx, RG
       /* object exists, but policy is broken */
       RGWBucketInfo info;
       RGWUserInfo uinfo;
-      string name;
       int r = store->get_bucket_info(ctx, obj.bucket.name, info);
       if (r < 0)
         goto done;
@@ -287,6 +286,7 @@ int rgw_build_policies(RGWRados *store, struct req_state *s, bool only_bucket, b
 {
   int ret = 0;
   string obj_str;
+  RGWUserInfo bucket_owner_info;
 
   s->bucket_acl = new RGWAccessControlPolicy(s->cct);
 
@@ -298,11 +298,12 @@ int rgw_build_policies(RGWRados *store, struct req_state *s, bool only_bucket, b
       return ret;
     }
     s->bucket = bucket_info.bucket;
-    s->bucket_owner = bucket_info.owner;
 
     string no_obj;
     RGWAccessControlPolicy bucket_acl(s->cct);
     ret = read_policy(store, s, bucket_info, s->bucket_acl, s->bucket, no_obj);
+
+    s->bucket_owner = s->bucket_acl->get_owner();
   }
 
   /* we're passed only_bucket = true when we specifically need the bucket's
@@ -384,13 +385,13 @@ int RGWGetObj::read_user_manifest_part(rgw_bucket& bucket, RGWObjEnt& ent, RGWAc
     if (ret < 0)
       goto done_err;
 
-    len = bl.length();
+    off_t len = bl.length();
     cur_ofs += len;
     ofs += len;
     ret = 0;
     perfcounter->tinc(l_rgw_get_lat,
                       (ceph_clock_now(s->cct) - start_time));
-    send_response_data(bl);
+    send_response_data(bl, 0, len);
 
     start_time = ceph_clock_now(s->cct);
   }
@@ -524,14 +525,43 @@ int RGWGetObj::handle_user_manifest(const char *prefix)
   return 0;
 }
 
+class RGWGetObj_CB : public RGWGetDataCB
+{
+  RGWGetObj *op;
+public:
+  RGWGetObj_CB(RGWGetObj *_op) : op(_op) {}
+  virtual ~RGWGetObj_CB() {}
+  
+  int handle_data(bufferlist& bl, off_t bl_ofs, off_t bl_len) {
+    return op->get_data_cb(bl, bl_ofs, bl_len);
+  }
+};
+
+int RGWGetObj::get_data_cb(bufferlist& bl, off_t bl_ofs, off_t bl_len)
+{
+  /* garbage collection related handling */
+  utime_t start_time = ceph_clock_now(s->cct);
+  if (start_time > gc_invalidate_time) {
+    int r = store->defer_gc(s->obj_ctx, obj);
+    if (r < 0) {
+      dout(0) << "WARNING: could not defer gc entry for obj" << dendl;
+    }
+    gc_invalidate_time = start_time;
+    gc_invalidate_time += (s->cct->_conf->rgw_gc_obj_min_wait / 2);
+  }
+  return send_response_data(bl, bl_ofs, bl_len);
+}
+
 void RGWGetObj::execute()
 {
   void *handle = NULL;
   utime_t start_time = s->time;
   bufferlist bl;
-  utime_t gc_invalidate_time = ceph_clock_now(s->cct);
+  gc_invalidate_time = ceph_clock_now(s->cct);
   gc_invalidate_time += (s->cct->_conf->rgw_gc_obj_min_wait / 2);
 
+  RGWGetObj_CB cb(this);
+
   map<string, bufferlist>::iterator attr_iter;
 
   perfcounter->inc(l_rgw_get);
@@ -539,11 +569,11 @@ void RGWGetObj::execute()
 
   ret = get_params();
   if (ret < 0)
-    goto done;
+    goto done_err;
 
   ret = init_common();
   if (ret < 0)
-    goto done;
+    goto done_err;
 
   new_ofs = ofs;
   new_end = end;
@@ -551,7 +581,7 @@ void RGWGetObj::execute()
   ret = store->prepare_get_obj(s->obj_ctx, obj, &new_ofs, &new_end, &attrs, mod_ptr,
                                unmod_ptr, &lastmod, if_match, if_nomatch, &total_len, &s->obj_size, &handle, &s->err);
   if (ret < 0)
-    goto done;
+    goto done_err;
 
   attr_iter = attrs.find(RGW_ATTR_USER_MANIFEST);
   if (attr_iter != attrs.end()) {
@@ -568,53 +598,22 @@ void RGWGetObj::execute()
   start = ofs;
 
   if (!get_data || ofs > end)
-    goto done;
+    goto done_err;
 
   perfcounter->inc(l_rgw_get_b, end - ofs);
 
-  while (ofs <= end) {
-    ret = store->get_obj(s->obj_ctx, &handle, obj, bl, ofs, end);
-    if (ret < 0) {
-      goto done;
-    }
-    len = ret;
+  ret = store->get_obj_iterate(s->obj_ctx, &handle, obj, ofs, end, &cb);
 
-    if (!len) {
-      dout(0) << "WARNING: failed to read object, returned zero length" << dendl;
-      ret = -EIO;
-      goto done;
-    }
-
-    ofs += len;
-    ret = 0;
-
-    perfcounter->tinc(l_rgw_get_lat,
-                     (ceph_clock_now(s->cct) - start_time));
-    ret = send_response_data(bl);
-    bl.clear();
-    if (ret < 0) {
-      dout(0) << "NOTICE: failed to send response to client" << dendl;
-      goto done;
-    }
-
-    start_time = ceph_clock_now(s->cct);
-
-    if (ofs <= end) {
-      if (start_time > gc_invalidate_time) {
-	int r = store->defer_gc(s->obj_ctx, obj);
-	if (r < 0) {
-	  dout(0) << "WARNING: could not defer gc entry for obj" << dendl;
-	}
-	gc_invalidate_time = start_time;
-        gc_invalidate_time += (s->cct->_conf->rgw_gc_obj_min_wait / 2);
-      }
-    }
+  perfcounter->tinc(l_rgw_get_lat,
+                   (ceph_clock_now(s->cct) - start_time));
+  if (ret < 0) {
+    goto done_err;
   }
 
-  return;
+  store->finish_get_obj(&handle);
 
-done:
-  send_response_data(bl);
+done_err:
+  send_response_data(bl, 0, 0);
   store->finish_get_obj(&handle);
 }
 
@@ -773,7 +772,7 @@ void RGWListBucket::execute()
 
 int RGWGetBucketLogging::verify_permission()
 {
-  if (s->user.user_id.compare(s->bucket_owner) != 0)
+  if (s->user.user_id.compare(s->bucket_owner.get_id()) != 0)
     return -EACCES;
 
   return 0;
@@ -811,7 +810,9 @@ void RGWCreateBucket::execute()
   if (ret < 0)
     return;
 
-  s->bucket_owner = s->user.user_id;
+  s->bucket_owner.set_id(s->user.user_id);
+  s->bucket_owner.set_name(s->user.display_name);
+
   r = get_policy_from_attr(s->cct, store, s->obj_ctx, &old_policy, obj);
   if (r >= 0)  {
     if (old_policy.get_owner().get_id().compare(s->user.user_id) != 0) {
@@ -1025,7 +1026,7 @@ int RGWPutObjProcessor_Aio::wait_pending_front()
 
 bool RGWPutObjProcessor_Aio::pending_has_completed()
 {
-  if (pending.size() == 0)
+  if (pending.empty())
     return false;
 
   struct put_obj_aio_info& info = pending.front();
@@ -1117,7 +1118,6 @@ int RGWPutObjProcessor_Atomic::prepare(RGWRados *store, struct req_state *s)
 {
   RGWPutObjProcessor::prepare(store, s);
 
-  string oid = s->object_str;
   head_obj.init(s->bucket, s->object_str);
 
   char buf[33];
@@ -1794,8 +1794,9 @@ void RGWPutACLs::execute()
     ret = -EINVAL;
     return;
   }
-  if (!s->canned_acl.empty()) {
-    ret = get_canned_policy(owner, ss);
+
+  if (!s->canned_acl.empty() || s->has_acl_header) {
+    ret = get_policy_from_state(store, s, ss);
     if (ret < 0)
       return;
 
@@ -1805,12 +1806,11 @@ void RGWPutACLs::execute()
     len = ss.str().size();
   }
 
-
   if (!parser.parse(data, len, 1)) {
     ret = -EACCES;
     return;
   }
-  policy = (RGWAccessControlPolicy_S3 *)parser.find_first("AccessControlPolicy");
+  policy = static_cast<RGWAccessControlPolicy_S3 *>(parser.find_first("AccessControlPolicy"));
   if (!policy) {
     ret = -EINVAL;
     return;
@@ -1982,7 +1982,7 @@ void RGWCompleteMultipart::execute()
     return;
   }
 
-  parts = (RGWMultiCompleteUpload *)parser.find_first("CompleteMultipartUpload");
+  parts = static_cast<RGWMultiCompleteUpload *>(parser.find_first("CompleteMultipartUpload"));
   if (!parts) {
     ret = -EINVAL;
     return;
@@ -2089,7 +2089,6 @@ void RGWAbortMultipart::execute()
   ret = -EINVAL;
   string upload_id;
   string meta_oid;
-  string prefix;
   upload_id = s->args.get("uploadId");
   map<uint32_t, RGWUploadPartInfo> obj_parts;
   map<uint32_t, RGWUploadPartInfo>::iterator obj_iter;
@@ -2193,7 +2192,7 @@ void RGWListBucketMultiparts::execute()
   marker_meta = marker.get_meta();
   ret = store->list_objects(s->bucket, max_uploads, prefix, delimiter, marker_meta, objs, common_prefixes,
                                !!(s->prot_flags & RGW_REST_SWIFT), mp_ns, &is_truncated, &mp_filter);
-  if (objs.size()) {
+  if (!objs.empty()) {
     vector<RGWObjEnt>::iterator iter;
     RGWMultipartUploadEntry entry;
     for (iter = objs.begin(); iter != objs.end(); ++iter) {
@@ -2243,7 +2242,7 @@ void RGWDeleteMultiObj::execute()
     goto error;
   }
 
-  multi_delete = (RGWMultiDelDelete *)parser.find_first("Delete");
+  multi_delete = static_cast<RGWMultiDelDelete *>(parser.find_first("Delete"));
   if (!multi_delete) {
     ret = -EINVAL;
     goto error;
@@ -2253,7 +2252,7 @@ void RGWDeleteMultiObj::execute()
     quiet = true;
 
   begin_response();
-  if (multi_delete->objects.size() == 0) {
+  if (multi_delete->objects.empty()) {
     goto done;
   }
 
