@@ -1,7 +1,9 @@
 #include <errno.h>
+#include <limits.h>
 
 #include "common/Formatter.h"
 #include "common/utf8.h"
+#include "include/str_list.h"
 #include "rgw_common.h"
 #include "rgw_rados.h"
 #include "rgw_formats.h"
@@ -11,94 +13,179 @@
 #include "rgw_rest_s3.h"
 #include "rgw_swift_auth.h"
 
-#include "rgw_formats.h"
-
-#ifdef FASTCGI_INCLUDE_DIR
-# include "fastcgi/fcgiapp.h"
-#else
-# include "fcgiapp.h"
-#endif
+#include "rgw_client_io.h"
+#include "rgw_resolve.h"
 
 #define dout_subsys ceph_subsys_rgw
 
+
+struct rgw_http_attr {
+  const char *rgw_attr;
+  const char *http_attr;
+};
+
+/*
+ * mapping between rgw object attrs and output http fields
+ */
+static struct rgw_http_attr rgw_to_http_attr_list[] = {
+  { RGW_ATTR_CONTENT_TYPE, "Content-Type"},
+  { RGW_ATTR_CONTENT_LANG, "Content-Language"},
+  { RGW_ATTR_EXPIRES, "Expires"},
+  { RGW_ATTR_CACHE_CONTROL, "Cache-Control"},
+  { RGW_ATTR_CONTENT_DISP, "Content-Disposition"},
+  { RGW_ATTR_CONTENT_ENC, "Content-Encoding"},
+  { NULL, NULL},
+};
+
+
+struct generic_attr {
+  const char *http_header;
+  const char *rgw_attr;
+};
+
+/*
+ * mapping between http env fields and rgw object attrs
+ */
+struct generic_attr generic_attrs[] = {
+  { "CONTENT_TYPE", RGW_ATTR_CONTENT_TYPE },
+  { "HTTP_CONTENT_LANGUAGE", RGW_ATTR_CONTENT_LANG },
+  { "HTTP_EXPIRES", RGW_ATTR_EXPIRES },
+  { "HTTP_CACHE_CONTROL", RGW_ATTR_CACHE_CONTROL },
+  { "HTTP_CONTENT_DISPOSITION", RGW_ATTR_CONTENT_DISP },
+  { "HTTP_CONTENT_ENCODING", RGW_ATTR_CONTENT_ENC },
+  { NULL, NULL },
+};
+
+map<string, string> rgw_to_http_attrs;
+static map<string, string> generic_attrs_map;
+
+/*
+ * make attrs look_like_this
+ */
+string lowercase_http_attr(const string& orig)
+{
+  const char *s = orig.c_str();
+  char buf[orig.size() + 1];
+  buf[orig.size()] = '\0';
+
+  for (size_t i = 0; i < orig.size(); ++i, ++s) {
+    switch (*s) {
+      case '-':
+	buf[i] = '_';
+	break;
+      default:
+	buf[i] = tolower(*s);
+    }
+  }
+  return string(buf);
+}
+
+/*
+ * make attrs LOOK_LIKE_THIS
+ */
+string uppercase_http_attr(const string& orig)
+{
+  const char *s = orig.c_str();
+  char buf[orig.size() + 1];
+  buf[orig.size()] = '\0';
+
+  for (size_t i = 0; i < orig.size(); ++i, ++s) {
+    switch (*s) {
+      case '-':
+	buf[i] = '_';
+	break;
+      default:
+	buf[i] = toupper(*s);
+    }
+  }
+  return string(buf);
+}
+
+/*
+ * make attrs Look-Like-This
+ */
+string camelcase_dash_http_attr(const string& orig)
+{
+  const char *s = orig.c_str();
+  char buf[orig.size() + 1];
+  buf[orig.size()] = '\0';
+
+  bool last_sep = true;
+
+  for (size_t i = 0; i < orig.size(); ++i, ++s) {
+    switch (*s) {
+      case '_':
+	buf[i] = '-';
+	last_sep = true;
+	break;
+      default:
+	if (last_sep)
+	  buf[i] = toupper(*s);
+	else
+	  buf[i] = tolower(*s);
+	last_sep = false;
+    }
+  }
+  return string(buf);
+}
+
+void rgw_rest_init(CephContext *cct)
+{
+  for (struct rgw_http_attr *attr = rgw_to_http_attr_list; attr->rgw_attr; attr++) {
+    rgw_to_http_attrs[attr->rgw_attr] = attr->http_attr;
+  }
+
+  for (struct generic_attr *gen_attr = generic_attrs; gen_attr->http_header; gen_attr++) {
+    generic_attrs_map[gen_attr->http_header] = gen_attr->rgw_attr;
+  }
+
+  list<string> extended_http_attrs;
+  get_str_list(cct->_conf->rgw_extended_http_attrs, extended_http_attrs);
+
+  list<string>::iterator iter;
+  for (iter = extended_http_attrs.begin(); iter != extended_http_attrs.end(); ++iter) {
+    string rgw_attr = RGW_ATTR_PREFIX;
+    rgw_attr.append(lowercase_http_attr(*iter));
+
+    rgw_to_http_attrs[rgw_attr] = camelcase_dash_http_attr(*iter);
+
+    string http_header = "HTTP_";
+    http_header.append(uppercase_http_attr(*iter));
+
+    generic_attrs_map[http_header] = rgw_attr;
+  }
+}
+
 static void dump_status(struct req_state *s, const char *status)
 {
-  CGI_PRINTF(s,"Status: %s\n", status);
-}
-
-struct rgw_html_errors {
-  int err_no;
-  int http_ret;
-  const char *s3_code;
-};
-
-const static struct rgw_html_errors RGW_HTML_ERRORS[] = {
-    { 0, 200, "" },
-    { STATUS_CREATED, 201, "Created" },
-    { STATUS_ACCEPTED, 202, "Accepted" },
-    { STATUS_NO_CONTENT, 204, "NoContent" },
-    { STATUS_PARTIAL_CONTENT, 206, "" },
-    { ERR_NOT_MODIFIED, 304, "NotModified" },
-    { EINVAL, 400, "InvalidArgument" },
-    { ERR_INVALID_REQUEST, 400, "InvalidRequest" },
-    { ERR_INVALID_DIGEST, 400, "InvalidDigest" },
-    { ERR_BAD_DIGEST, 400, "BadDigest" },
-    { ERR_INVALID_BUCKET_NAME, 400, "InvalidBucketName" },
-    { ERR_INVALID_OBJECT_NAME, 400, "InvalidObjectName" },
-    { ERR_UNRESOLVABLE_EMAIL, 400, "UnresolvableGrantByEmailAddress" },
-    { ERR_INVALID_PART, 400, "InvalidPart" },
-    { ERR_INVALID_PART_ORDER, 400, "InvalidPartOrder" },
-    { ERR_REQUEST_TIMEOUT, 400, "RequestTimeout" },
-    { ERR_TOO_LARGE, 400, "EntityTooLarge" },
-    { ERR_TOO_MANY_BUCKETS, 400, "TooManyBuckets" },
-    { ERR_LENGTH_REQUIRED, 411, "MissingContentLength" },
-    { EACCES, 403, "AccessDenied" },
-    { EPERM, 403, "AccessDenied" },
-    { ERR_USER_SUSPENDED, 403, "UserSuspended" },
-    { ERR_REQUEST_TIME_SKEWED, 403, "RequestTimeTooSkewed" },
-    { ENOENT, 404, "NoSuchKey" },
-    { ERR_NO_SUCH_BUCKET, 404, "NoSuchBucket" },
-    { ERR_NO_SUCH_UPLOAD, 404, "NoSuchUpload" },
-    { ERR_METHOD_NOT_ALLOWED, 405, "MethodNotAllowed" },
-    { ETIMEDOUT, 408, "RequestTimeout" },
-    { EEXIST, 409, "BucketAlreadyExists" },
-    { ENOTEMPTY, 409, "BucketNotEmpty" },
-    { ERR_PRECONDITION_FAILED, 412, "PreconditionFailed" },
-    { ERANGE, 416, "InvalidRange" },
-    { ERR_UNPROCESSABLE_ENTITY, 422, "UnprocessableEntity" },
-    { ERR_INTERNAL_ERROR, 500, "InternalError" },
-};
-
-const static struct rgw_html_errors RGW_HTML_SWIFT_ERRORS[] = {
-    { EACCES, 401, "AccessDenied" },
-    { EPERM, 401, "AccessDenied" },
-    { ERR_USER_SUSPENDED, 401, "UserSuspended" },
-    { ERR_INVALID_UTF8, 412, "Invalid UTF8" },
-    { ERR_BAD_URL, 412, "Bad URL" },
-};
-
-#define ARRAY_LEN(arr) (sizeof(arr) / sizeof(arr[0]))
-
-static const struct rgw_html_errors *search_err(int err_no, const struct rgw_html_errors *errs, int len)
-{
-  for (int i = 0; i < len; ++i, ++errs) {
-    if (err_no == errs->err_no)
-      return errs;
+  int r = s->cio->print("Status: %s\n", status);
+  if (r < 0) {
+    ldout(s->cct, 0) << "ERROR: s->cio->print() returned err=" << r << dendl;
   }
-  return NULL;
 }
 
-void flush_formatter_to_req_state(struct req_state *s, Formatter *formatter)
+void rgw_flush_formatter_and_reset(struct req_state *s, Formatter *formatter)
 {
   std::ostringstream oss;
   formatter->flush(oss);
   std::string outs(oss.str());
   if (!outs.empty()) {
-    CGI_PutStr(s, outs.c_str(), outs.size());
+    s->cio->write(outs.c_str(), outs.size());
   }
+
   s->formatter->reset();
 }
 
+void rgw_flush_formatter(struct req_state *s, Formatter *formatter)
+{
+  std::ostringstream oss;
+  formatter->flush(oss);
+  std::string outs(oss.str());
+  if (!outs.empty()) {
+    s->cio->write(outs.c_str(), outs.size());
+  }
+}
+
 void set_req_state_err(struct req_state *s, int err_no)
 {
   const struct rgw_html_errors *r;
@@ -140,34 +227,96 @@ void dump_errno(struct req_state *s, int err)
   dump_status(s, buf);
 }
 
-void dump_content_length(struct req_state *s, size_t len)
+void dump_content_length(struct req_state *s, uint64_t len)
 {
-  char buf[16];
-  snprintf(buf, sizeof(buf), "%lu", (long unsigned int)len);
-  CGI_PRINTF(s, "Content-Length: %s\n", buf);
-  CGI_PRINTF(s, "Accept-Ranges: %s\n", "bytes");
+  char buf[21];
+  snprintf(buf, sizeof(buf), "%"PRIu64, len);
+  int r = s->cio->print("Content-Length: %s\n", buf);
+  if (r < 0) {
+    ldout(s->cct, 0) << "ERROR: s->cio->print() returned err=" << r << dendl;
+  }
+  r = s->cio->print("Accept-Ranges: %s\n", "bytes");
+  if (r < 0) {
+    ldout(s->cct, 0) << "ERROR: s->cio->print() returned err=" << r << dendl;
+  }
 }
 
 void dump_etag(struct req_state *s, const char *etag)
 {
+  int r;
   if (s->prot_flags & RGW_REST_SWIFT)
-    CGI_PRINTF(s,"etag: %s\n", etag);
+    r = s->cio->print("etag: %s\n", etag);
   else
-    CGI_PRINTF(s,"ETag: \"%s\"\n", etag);
+    r = s->cio->print("ETag: \"%s\"\n", etag);
+  if (r < 0) {
+    ldout(s->cct, 0) << "ERROR: s->cio->print() returned err=" << r << dendl;
+  }
+}
+
+void dump_pair(struct req_state *s, const char *key, const char *value)
+{
+  if ( (strlen(key) > 0) && (strlen(value) > 0))
+    s->cio->print("%s: %s\n", key, value);
+}
+
+void dump_bucket_from_state(struct req_state *s)
+{
+  if (!s->bucket_name_str.empty())
+    s->cio->print("Bucket: \"%s\"\n", s->bucket_name_str.c_str());
+}
+
+void dump_object_from_state(struct req_state *s)
+{
+  if (!s->object_str.empty())
+    s->cio->print("Key: \"%s\"\n", s->object_str.c_str());
+}
+
+void dump_uri_from_state(struct req_state *s)
+{
+  if (strcmp(s->request_uri.c_str(), "/") == 0) {
+
+    string location = "http://";
+    string server = s->env->get("SERVER_NAME", "<SERVER_NAME>");
+    location.append(server);
+    location += "/";
+    if (!s->bucket_name_str.empty()) {
+      location += s->bucket_name_str;
+      location += "/";
+      if (!s->object_str.empty()) {
+        location += s->object_str;
+        s->cio->print("Location: %s\n", location.c_str());
+      }
+    }
+  }
+  else {
+    s->cio->print("Location: \"%s\"\n", s->request_uri.c_str());
+  }
+}
+
+void dump_redirect(struct req_state *s, const string& redirect)
+{
+  if (redirect.empty())
+    return;
+
+  s->cio->print("Location: %s\n", redirect.c_str());
 }
 
 void dump_last_modified(struct req_state *s, time_t t)
 {
 
   char timestr[TIME_BUF_SIZE];
-  struct tm *tmp = gmtime(&t);
+  struct tm result;
+  struct tm *tmp = gmtime_r(&t, &result);
   if (tmp == NULL)
     return;
 
   if (strftime(timestr, sizeof(timestr), "%a, %d %b %Y %H:%M:%S %Z", tmp) == 0)
     return;
 
-  CGI_PRINTF(s, "Last-Modified: %s\n", timestr);
+  int r = s->cio->print("Last-Modified: %s\n", timestr);
+  if (r < 0) {
+    ldout(s->cct, 0) << "ERROR: s->cio->print() returned err=" << r << dendl;
+  }
 }
 
 void dump_time(struct req_state *s, const char *name, time_t *t)
@@ -233,24 +382,31 @@ void end_header(struct req_state *s, const char *content_type)
     s->formatter->close_section();
     dump_content_length(s, s->formatter->get_len());
   }
-  CGI_PRINTF(s,"Content-type: %s\r\n\r\n", content_type);
-  s->header_ended = true;
-  flush_formatter_to_req_state(s, s->formatter);
+  int r = s->cio->print("Content-type: %s\r\n\r\n", content_type);
+  if (r < 0) {
+    ldout(s->cct, 0) << "ERROR: s->cio->print() returned err=" << r << dendl;
+  }
+  s->cio->set_account(true);
+  rgw_flush_formatter_and_reset(s, s->formatter);
 }
 
 void abort_early(struct req_state *s, int err_no)
 {
+  if (!s->formatter) {
+    s->formatter = new JSONFormatter;
+    s->format = RGW_FORMAT_JSON;
+  }
   set_req_state_err(s, err_no);
   dump_errno(s);
   end_header(s);
-  flush_formatter_to_req_state(s, s->formatter);
+  rgw_flush_formatter_and_reset(s, s->formatter);
   perfcounter->inc(l_rgw_failed_req);
 }
 
 void dump_continue(struct req_state *s)
 {
   dump_status(s, "100");
-  FCGX_FFlush(s->fcgx->out);
+  s->cio->flush();
 }
 
 void dump_range(struct req_state *s, uint64_t ofs, uint64_t end, uint64_t total)
@@ -259,10 +415,13 @@ void dump_range(struct req_state *s, uint64_t ofs, uint64_t end, uint64_t total)
 
   /* dumping range into temp buffer first, as libfcgi will fail to digest %lld */
   snprintf(range_buf, sizeof(range_buf), "%lld-%lld/%lld", (long long)ofs, (long long)end, (long long)total);
-  CGI_PRINTF(s,"Content-Range: bytes %s\n", range_buf);
+  int r = s->cio->print("Content-Range: bytes %s\n", range_buf);
+  if (r < 0) {
+    ldout(s->cct, 0) << "ERROR: s->cio->print() returned err=" << r << dendl;
+  }
 }
 
-int RGWGetObj_REST::get_params()
+int RGWGetObj_ObjStore::get_params()
 {
   range_str = s->env->get("HTTP_RANGE");
   if_mod = s->env->get("HTTP_IF_MODIFIED_SINCE");
@@ -273,8 +432,154 @@ int RGWGetObj_REST::get_params()
   return 0;
 }
 
+int RESTArgs::get_string(struct req_state *s, const string& name, const string& def_val, string *val, bool *existed)
+{
+  bool exists;
+  *val = s->args.get(name, &exists);
 
-int RGWPutObj_REST::verify_params()
+  if (existed)
+    *existed = exists;
+
+  if (!exists) {
+    *val = def_val;
+    return 0;
+  }
+
+  return 0;
+}
+
+int RESTArgs::get_uint64(struct req_state *s, const string& name, uint64_t def_val, uint64_t *val, bool *existed)
+{
+  bool exists;
+  string sval = s->args.get(name, &exists);
+
+  if (existed)
+    *existed = exists;
+
+  if (!exists) {
+    *val = def_val;
+    return 0;
+  }
+
+  int r = stringtoull(sval, val);
+  if (r < 0)
+    return r;
+
+  return 0;
+}
+
+int RESTArgs::get_int64(struct req_state *s, const string& name, int64_t def_val, int64_t *val, bool *existed)
+{
+  bool exists;
+  string sval = s->args.get(name, &exists);
+
+  if (existed)
+    *existed = exists;
+
+  if (!exists) {
+    *val = def_val;
+    return 0;
+  }
+
+  int r = stringtoll(sval, val);
+  if (r < 0)
+    return r;
+
+  return 0;
+}
+
+int RESTArgs::get_time(struct req_state *s, const string& name, const utime_t& def_val, utime_t *val, bool *existed)
+{
+  bool exists;
+  string sval = s->args.get(name, &exists);
+
+  if (existed)
+    *existed = exists;
+
+  if (!exists) {
+    *val = def_val;
+    return 0;
+  }
+
+  uint64_t epoch;
+
+  int r = parse_date(sval, &epoch);
+  if (r < 0)
+    return r;
+
+  *val = utime_t(epoch, 0);
+
+  return 0;
+}
+
+int RESTArgs::get_epoch(struct req_state *s, const string& name, uint64_t def_val, uint64_t *epoch, bool *existed)
+{
+  bool exists;
+  string date = s->args.get(name, &exists);
+
+  if (existed)
+    *existed = exists;
+
+  if (!exists) {
+    *epoch = def_val;
+    return 0;
+  }
+
+  int r = parse_date(date, epoch);
+  if (r < 0)
+    return r;
+
+  return 0;
+}
+
+int RESTArgs::get_bool(struct req_state *s, const string& name, bool def_val, bool *val, bool *existed)
+{
+  bool exists;
+  string sval = s->args.get(name, &exists);
+
+  if (existed)
+    *existed = exists;
+
+  if (!exists) {
+    *val = def_val;
+    return 0;
+  }
+
+  const char *str = sval.c_str();
+
+  if (sval.empty() ||
+      strcasecmp(str, "true") == 0 ||
+      sval.compare("1") == 0) {
+    *val = true;
+    return 0;
+  }
+
+  if (strcasecmp(str, "false") != 0 &&
+      sval.compare("0") != 0) {
+    *val = def_val;
+    return -EINVAL;
+  }
+
+  *val = false;
+  return 0;
+}
+
+
+void RGWRESTFlusher::do_start(int ret)
+{
+  set_req_state_err(s, ret); /* no going back from here */
+  dump_errno(s);
+  dump_start(s);
+  end_header(s);
+  rgw_flush_formatter_and_reset(s, s->formatter);
+}
+
+void RGWRESTFlusher::do_flush()
+{
+  rgw_flush_formatter(s, s->formatter);
+}
+
+int RGWPutObj_ObjStore::verify_params()
 {
   if (s->length) {
     off_t len = atoll(s->length);
@@ -286,14 +591,14 @@ int RGWPutObj_REST::verify_params()
   return 0;
 }
 
-int RGWPutObj_REST::get_params()
+int RGWPutObj_ObjStore::get_params()
 {
   supplied_md5_b64 = s->env->get("HTTP_CONTENT_MD5");
 
   return 0;
 }
 
-int RGWPutObj_REST::get_data(bufferlist& bl)
+int RGWPutObj_ObjStore::get_data(bufferlist& bl)
 {
   size_t cl;
   if (s->length) {
@@ -308,8 +613,12 @@ int RGWPutObj_REST::get_data(bufferlist& bl)
   if (cl) {
     bufferptr bp(cl);
 
-    CGI_GetStr(s, bp.c_str(), cl, len);
-    bl.append(bp);
+    int read_len; /* cio->read() expects int * */
+    int r = s->cio->read(bp.c_str(), cl, &read_len);
+    len = read_len;
+    if (r < 0)
+      return ret;
+    bl.append(bp, 0, len);
   }
 
   if ((uint64_t)ofs + len > RGW_MAX_PUT_SIZE) {
@@ -322,7 +631,23 @@ int RGWPutObj_REST::get_data(bufferlist& bl)
   return len;
 }
 
-int RGWPutACLs_REST::get_params()
+int RGWPostObj_ObjStore::verify_params()
+{
+  /*  check that we have enough memory to store the object
+  note that this test isn't exact and may fail unintentionally
+  for large requests is */
+  if (!s->length) {
+    return -ERR_LENGTH_REQUIRED;
+  }
+  off_t len = atoll(s->length);
+  if (len > (off_t)RGW_MAX_PUT_SIZE) {
+    return -ERR_TOO_LARGE;
+  }
+
+  return 0;
+}
+
+int RGWPutACLs_ObjStore::get_params()
 {
   size_t cl = 0;
   if (s->length)
@@ -333,7 +658,11 @@ int RGWPutACLs_REST::get_params()
        ret = -ENOMEM;
        return ret;
     }
-    CGI_GetStr(s, data, cl, len);
+    int read_len;
+    int r = s->cio->read(data, cl, &read_len);
+    len = read_len;
+    if (r < 0)
+      return r;
     data[len] = '\0';
   } else {
     len = 0;
@@ -342,15 +671,6 @@ int RGWPutACLs_REST::get_params()
   return ret;
 }
 
-int RGWInitMultipart_REST::get_params()
-{
-  if (!s->args.exists("uploads")) {
-    ret = -ENOTSUP;
-  }
-
-  return ret;
-}
-
 static int read_all_chunked_input(req_state *s, char **pdata, int *plen)
 {
 #define READ_CHUNK 4096
@@ -363,7 +683,11 @@ static int read_all_chunked_input(req_state *s, char **pdata, int *plen)
 
   int read_len = 0, len = 0;
   do {
-    CGI_GetStr(s, data + len, need_to_read, read_len);
+    int r = s->cio->read(data + len, need_to_read, &read_len);
+    if (r < 0) {
+      free(data);
+      return r;
+    }
 
     len += read_len;
 
@@ -392,7 +716,7 @@ static int read_all_chunked_input(req_state *s, char **pdata, int *plen)
   return 0;
 }
 
-int RGWCompleteMultipart_REST::get_params()
+int RGWCompleteMultipart_ObjStore::get_params()
 {
   upload_id = s->args.get("uploadId");
 
@@ -410,7 +734,9 @@ int RGWCompleteMultipart_REST::get_params()
        ret = -ENOMEM;
        return ret;
     }
-    CGI_GetStr(s, data, cl, len);
+    ret = s->cio->read(data, cl, &len);
+    if (ret < 0)
+      return ret;
     data[len] = '\0';
   } else {
     const char *encoding = s->env->get("HTTP_TRANSFER_ENCODING");
@@ -425,7 +751,7 @@ int RGWCompleteMultipart_REST::get_params()
   return ret;
 }
 
-int RGWListMultipart_REST::get_params()
+int RGWListMultipart_ObjStore::get_params()
 {
   upload_id = s->args.get("uploadId");
 
@@ -443,7 +769,7 @@ int RGWListMultipart_REST::get_params()
   return ret;
 }
 
-int RGWListBucketMultiparts_REST::get_params()
+int RGWListBucketMultiparts_ObjStore::get_params()
 {
   delimiter = s->args.get("delimiter");
   prefix = s->args.get("prefix");
@@ -461,171 +787,55 @@ int RGWListBucketMultiparts_REST::get_params()
   return 0;
 }
 
-static void next_tok(string& str, string& tok, char delim)
+int RGWDeleteMultiObj_ObjStore::get_params()
 {
-  if (str.size() == 0) {
-    tok = "";
-    return;
+  bucket_name = s->bucket_name;
+
+  if (bucket_name.empty()) {
+    ret = -EINVAL;
+    return ret;
   }
-  tok = str;
-  int pos = str.find(delim);
-  if (pos > 0) {
-    tok = str.substr(0, pos);
-    str = str.substr(pos + 1);
+
+  // everything is probably fine, set the bucket
+  bucket = s->bucket;
+
+  size_t cl = 0;
+
+  if (s->length)
+    cl = atoll(s->length);
+  if (cl) {
+    data = (char *)malloc(cl + 1);
+    if (!data) {
+      ret = -ENOMEM;
+      return ret;
+    }
+    int read_len;
+    ret = s->cio->read(data, cl, &read_len);
+    len = read_len;
+    if (ret < 0)
+      return ret;
+    data[len] = '\0';
   } else {
-    str = "";
+    return -EINVAL;
   }
+
+  return ret;
 }
 
-static int init_entities_from_header(struct req_state *s)
+
+void RGWRESTOp::send_response()
 {
-  string req;
-  string first;
-
-  s->bucket_name = NULL;
-  s->bucket.clear();
-  s->object = NULL;
-  s->object_str = "";
-
-  s->header_ended = false;
-  s->bytes_sent = 0;
-  s->bytes_received = 0;
-  s->obj_size = 0;
-
-  /* this is the default, might change in a few lines */
-  s->format = RGW_FORMAT_XML;
-  s->formatter = new XMLFormatter(false);
-
-  int pos;
-  if (g_conf->rgw_dns_name.length() && s->host) {
-    string h(s->host);
-
-    dout(10) << "host=" << s->host << " rgw_dns_name=" << g_conf->rgw_dns_name << dendl;
-    pos = h.find(g_conf->rgw_dns_name);
-
-    if (pos > 0 && h[pos - 1] == '.') {
-      string encoded_bucket = h.substr(0, pos-1);
-      s->bucket_name_str = encoded_bucket;
-      s->bucket_name = strdup(s->bucket_name_str.c_str());
-      s->host_bucket = s->bucket_name;
-    } else {
-      s->host_bucket = NULL;
-    }
-  } else
-    s->host_bucket = NULL;
-
-  const char *req_name = s->decoded_uri.c_str();
-  const char *p;
-
-  if (*req_name == '?') {
-    p = req_name;
-  } else {
-    p = s->request_params.c_str();
+  if (!flusher.did_start()) {
+    set_req_state_err(s, http_ret);
+    dump_errno(s);
+    end_header(s);
   }
+  flusher.flush();
+}
 
-  s->args.set(p);
-  s->args.parse();
-
-  if (*req_name != '/')
-    goto done;
-
-  req_name++;
-
-  if (!*req_name)
-    goto done;
-
-  req = req_name;
-
-  pos = req.find('/');
-  if (pos >= 0) {
-    bool cut_url = g_conf->rgw_swift_url_prefix.length();
-    first = req.substr(0, pos);
-    if (first.compare(g_conf->rgw_swift_url_prefix) == 0) {
-      s->prot_flags |= RGW_REST_SWIFT;
-      if (cut_url) {
-        next_tok(req, first, '/');
-      }
-    }
-  } else {
-    if (req.compare(g_conf->rgw_swift_url_prefix) == 0) {
-      s->prot_flags |= RGW_REST_SWIFT;
-      delete s->formatter;
-      s->format = 0;
-      s->formatter = new RGWFormatter_Plain;
-      return -ERR_BAD_URL;
-    }
-    first = req;
-  }
-
-  if (s->prot_flags & RGW_REST_SWIFT) {
-    /* verify that the request_uri conforms with what's expected */
-    char buf[g_conf->rgw_swift_url_prefix.length() + 16];
-    int blen = sprintf(buf, "/%s/v1", g_conf->rgw_swift_url_prefix.c_str());
-    if (s->decoded_uri[0] != '/' ||
-        s->decoded_uri.compare(0, blen, buf) !=  0) {
-      return -ENOENT;
-    }
-
-    s->format = 0;
-    delete s->formatter;
-    s->formatter = new RGWFormatter_Plain;
-    string format_str = s->args.get("format");
-    if (format_str.compare("xml") == 0) {
-      s->format = RGW_FORMAT_XML;
-      delete s->formatter;
-      s->formatter = new XMLFormatter(false);
-    } else if (format_str.compare("json") == 0) {
-      s->format = RGW_FORMAT_JSON;
-      delete s->formatter;
-      s->formatter = new JSONFormatter(false);
-    }
-  }
-
-  if (s->prot_flags & RGW_REST_SWIFT) {
-    string ver;
-
-    next_tok(req, ver, '/');
-    dout(10) << "ver=" << ver << dendl;
-    s->os_auth_token = s->env->get("HTTP_X_AUTH_TOKEN");
-    next_tok(req, first, '/');
-
-    dout(10) << "ver=" << ver << " first=" << first << " req=" << req << dendl;
-    if (first.size() == 0)
-      goto done;
-
-    s->bucket_name_str = first;
-    s->bucket_name = strdup(s->bucket_name_str.c_str());
-   
-    if (req.size()) {
-      s->object_str = req;
-      s->object = strdup(s->object_str.c_str());
-    }
-
-    goto done;
-  }
-  if (!s->bucket_name) {
-    s->bucket_name_str = first;
-    s->bucket_name = strdup(s->bucket_name_str.c_str());
-  } else {
-    s->object_str = req_name;
-    s->object = strdup(s->object_str.c_str());
-    goto done;
-  }
-
-  if (strcmp(s->bucket_name, "auth") == 0)
-    s->prot_flags |= RGW_REST_SWIFT_AUTH;
-
-  if (pos >= 0) {
-    string encoded_obj_str = req.substr(pos+1);
-    s->object_str = encoded_obj_str;
-
-    if (s->object_str.size() > 0) {
-      s->object = strdup(s->object_str.c_str());
-    }
-  }
-done:
-  s->formatter->reset();
-  return 0;
+int RGWRESTOp::verify_permission()
+{
+  return check_caps(s->user.caps);
 }
 
 static void line_unfold(const char *line, string& sdest)
@@ -675,13 +885,15 @@ struct str_len meta_prefixes[] = { STR_LEN_ENTRY("HTTP_X_AMZ"),
                                    STR_LEN_ENTRY("HTTP_X_CONTAINER"),
                                    {NULL, 0} };
 
-static int init_auth_info(struct req_state *s)
+static int init_meta_info(struct req_state *s)
 {
   const char *p;
 
   s->x_meta_map.clear();
 
-  for (int i=0; (p = s->fcgx->envp[i]); ++i) {
+  const char **envp = s->cio->envp();
+
+  for (int i=0; (p = envp[i]); ++i) {
     const char *prefix;
     for (int prefix_num = 0; (prefix = meta_prefixes[prefix_num].str) != NULL; prefix_num++) {
       int len = meta_prefixes[prefix_num].len;
@@ -732,34 +944,42 @@ static int init_auth_info(struct req_state *s)
   return 0;
 }
 
-static bool looks_like_ip_address(const char *bucket)
+int RGWHandler_ObjStore::allocate_formatter(struct req_state *s, int default_type, bool configurable)
 {
-  int num_periods = 0;
-  bool expect_period = false;
-  for (const char *b = bucket; *b; ++b) {
-    if (*b == '.') {
-      if (!expect_period)
-	return false;
-      ++num_periods;
-      if (num_periods > 3)
-	return false;
-      expect_period = false;
-    }
-    else if (isdigit(*b)) {
-      expect_period = true;
-    }
-    else {
-      return false;
+  s->format = default_type;
+  if (configurable) {
+    string format_str = s->args.get("format");
+    if (format_str.compare("xml") == 0) {
+      s->format = RGW_FORMAT_XML;
+    } else if (format_str.compare("json") == 0) {
+      s->format = RGW_FORMAT_JSON;
     }
   }
-  return (num_periods == 3);
+
+  switch (s->format) {
+    case RGW_FORMAT_PLAIN:
+      s->formatter = new RGWFormatter_Plain;
+      break;
+    case RGW_FORMAT_XML:
+      s->formatter = new XMLFormatter(false);
+      break;
+    case RGW_FORMAT_JSON:
+      s->formatter = new JSONFormatter(false);
+      break;
+    default:
+      return -EINVAL;
+
+  };
+  s->formatter->reset();
+
+  return 0;
 }
 
 // This function enforces Amazon's spec for bucket names.
 // (The requirements, not the recommendations.)
-static int validate_bucket_name(const char *bucket, int flags)
+int RGWHandler_ObjStore::validate_bucket_name(const string& bucket)
 {
-  int len = strlen(bucket);
+  int len = bucket.size();
   if (len < 3) {
     if (len == 0) {
       // This request doesn't specify a bucket at all
@@ -773,40 +993,6 @@ static int validate_bucket_name(const char *bucket, int flags)
     return -ERR_INVALID_BUCKET_NAME;
   }
 
-  if (flags & RGW_REST_SWIFT) {
-    if (*bucket == '.')
-      return -ERR_INVALID_BUCKET_NAME;
-
-    if (check_utf8(bucket, len))
-      return -ERR_INVALID_UTF8;
-
-    for (int i = 0; i < len; ++i) {
-      if ((unsigned char)bucket[i] == 0xff)
-        return -ERR_INVALID_BUCKET_NAME;
-    }
-
-    return 0;
-  }
-
-  if (!(isalpha(bucket[0]) || isdigit(bucket[0]))) {
-    // bucket names must start with a number or letter
-    return -ERR_INVALID_BUCKET_NAME;
-  }
-
-  for (const char *s = bucket; *s; ++s) {
-    char c = *s;
-    if (isdigit(c) || (c == '.'))
-      continue;
-    if (isalpha(c))
-      continue;
-    if ((c == '-') || (c == '_'))
-      continue;
-    // Invalid character
-    return -ERR_INVALID_BUCKET_NAME;
-  }
-
-  if (looks_like_ip_address(bucket))
-    return -ERR_INVALID_BUCKET_NAME;
   return 0;
 }
 
@@ -814,98 +1000,42 @@ static int validate_bucket_name(const char *bucket, int flags)
 // is at most 1024 bytes long."
 // However, we can still have control characters and other nasties in there.
 // Just as long as they're utf-8 nasties.
-static int validate_object_name(const char *object)
+int RGWHandler_ObjStore::validate_object_name(const string& object)
 {
-  int len = strlen(object);
+  int len = object.size();
   if (len > 1024) {
     // Name too long
     return -ERR_INVALID_OBJECT_NAME;
   }
 
-  if (check_utf8(object, len)) {
+  if (check_utf8(object.c_str(), len)) {
     // Object names must be valid UTF-8.
     return -ERR_INVALID_OBJECT_NAME;
   }
   return 0;
 }
 
-int RGWHandler_REST::preprocess(struct req_state *s, FCGX_Request *fcgx)
+static http_op op_from_method(const char *method)
 {
-  int ret = 0;
+  if (!method)
+    return OP_UNKNOWN;
+  if (strcmp(method, "GET") == 0)
+    return OP_GET;
+  if (strcmp(method, "PUT") == 0)
+    return OP_PUT;
+  if (strcmp(method, "DELETE") == 0)
+    return OP_DELETE;
+  if (strcmp(method, "HEAD") == 0)
+    return OP_HEAD;
+  if (strcmp(method, "POST") == 0)
+    return OP_POST;
+  if (strcmp(method, "COPY") == 0)
+    return OP_COPY;
 
-  s->fcgx = fcgx;
-  s->request_uri = s->env->get("REQUEST_URI");
-  int pos = s->request_uri.find('?');
-  if (pos >= 0) {
-    s->request_params = s->request_uri.substr(pos + 1);
-    s->request_uri = s->request_uri.substr(0, pos);
-  }
-  url_decode(s->request_uri, s->decoded_uri);
-  s->method = s->env->get("REQUEST_METHOD");
-  s->host = s->env->get("HTTP_HOST");
-  s->length = s->env->get("CONTENT_LENGTH");
-  s->content_type = s->env->get("CONTENT_TYPE");
-  s->prot_flags = 0;
-
-  if (!s->method)
-    s->op = OP_UNKNOWN;
-  else if (strcmp(s->method, "GET") == 0)
-    s->op = OP_GET;
-  else if (strcmp(s->method, "PUT") == 0)
-    s->op = OP_PUT;
-  else if (strcmp(s->method, "DELETE") == 0)
-    s->op = OP_DELETE;
-  else if (strcmp(s->method, "HEAD") == 0)
-    s->op = OP_HEAD;
-  else if (strcmp(s->method, "POST") == 0)
-    s->op = OP_POST;
-  else if (strcmp(s->method, "COPY") == 0)
-    s->op = OP_COPY;
-  else
-    s->op = OP_UNKNOWN;
-
-  ret = init_entities_from_header(s);
-  if (ret)
-    return ret;
-
-  switch (s->op) {
-  case OP_PUT:
-    if (s->object && !s->args.sub_resource_exists("acl")) {
-      if (s->length && *s->length == '\0')
-        ret = -EINVAL;
-    }
-    if (s->length)
-      s->content_length = atoll(s->length);
-    else
-      s->content_length = 0;
-    break;
-  default:
-    break;
-  }
-
-  if (ret)
-    return ret;
-
-  ret = validate_bucket_name(s->bucket_name_str.c_str(), s->prot_flags);
-  if (ret)
-    return ret;
-  ret = validate_object_name(s->object_str.c_str());
-  if (ret)
-    return ret;
-  dout(10) << "s->object=" << (s->object ? s->object : "<NULL>") << " s->bucket=" << (s->bucket_name ? s->bucket_name : "<NULL>") << dendl;
-
-  init_auth_info(s);
-
-  s->http_auth = s->env->get("HTTP_AUTHORIZATION");
-
-  if (g_conf->rgw_print_continue) {
-    const char *expect = s->env->get("HTTP_EXPECT");
-    s->expect_cont = (expect && !strcasecmp(expect, "100-continue"));
-  }
-  return ret;
+  return OP_UNKNOWN;
 }
 
-int RGWHandler_REST::read_permissions(RGWOp *op_obj)
+int RGWHandler_ObjStore::read_permissions(RGWOp *op_obj)
 {
   bool only_bucket;
 
@@ -916,13 +1046,20 @@ int RGWHandler_REST::read_permissions(RGWOp *op_obj)
     break;
   case OP_PUT:
   case OP_POST:
+    /* is it a 'multi-object delete' request? */
+    if (s->request_params == "delete") {
+      only_bucket = true;
+      break;
+    }
     if (is_obj_update_op()) {
       only_bucket = false;
       break;
     }
     /* is it a 'create bucket' request? */
-    if (s->object_str.size() == 0)
+    if ((s->op == OP_PUT) && s->object_str.size() == 0)
       return 0;
+    only_bucket = true;
+    break;
   case OP_DELETE:
     only_bucket = true;
     break;
@@ -935,74 +1072,148 @@ int RGWHandler_REST::read_permissions(RGWOp *op_obj)
   return do_read_permissions(op_obj, only_bucket);
 }
 
-RGWOp *RGWHandler_REST::get_op()
+void RGWRESTMgr::register_resource(string resource, RGWRESTMgr *mgr)
 {
-  RGWOp *op;
-  switch (s->op) {
-   case OP_GET:
-     op = get_retrieve_op(true);
-     break;
-   case OP_PUT:
-     op = get_create_op();
-     break;
-   case OP_DELETE:
-     op = get_delete_op();
-     break;
-   case OP_HEAD:
-     op = get_retrieve_op(false);
-     break;
-   case OP_POST:
-     op = get_post_op();
-     break;
-   case OP_COPY:
-     op = get_copy_op();
-     break;
-   default:
-     return NULL;
-  }
-
-  if (op) {
-    op->init(s, this);
-  }
-  return op;
+  string r = "/";
+  r.append(resource);
+  resource_mgrs[r] = mgr;
+  resources_by_size.insert(pair<size_t, string>(r.size(), r));
 }
 
-
-RGWRESTMgr::RGWRESTMgr()
+void RGWRESTMgr::register_default_mgr(RGWRESTMgr *mgr)
 {
-  m_os_handler = new RGWHandler_REST_SWIFT;
-  m_os_auth_handler = new RGWHandler_SWIFT_Auth;
-  m_s3_handler = new RGWHandler_REST_S3;
+  delete default_mgr;
+  default_mgr = mgr;
+}
+
+RGWRESTMgr *RGWRESTMgr::get_resource_mgr(struct req_state *s, const string& uri)
+{
+  if (resources_by_size.empty())
+    return this;
+
+  multimap<size_t, string>::reverse_iterator iter;
+
+  for (iter = resources_by_size.rbegin(); iter != resources_by_size.rend(); ++iter) {
+    string& resource = iter->second;
+    if (uri.compare(0, iter->first, resource) == 0 &&
+	(resource.size() == iter->first ||
+	 resource[iter->first] == '/')) {
+      string suffix = uri.substr(iter->first);
+      return resource_mgrs[resource]->get_resource_mgr(s, suffix);
+    }
+  }
+
+  if (default_mgr)
+    return default_mgr;
+
+  return this;
 }
 
 RGWRESTMgr::~RGWRESTMgr()
 {
-  delete m_os_handler;
-  delete m_os_auth_handler;
-  delete m_s3_handler;
+  map<string, RGWRESTMgr *>::iterator iter;
+  for (iter = resource_mgrs.begin(); iter != resource_mgrs.end(); ++iter) {
+    delete iter->second;
+  }
+  delete default_mgr;
 }
 
-RGWHandler *RGWRESTMgr::get_handler(struct req_state *s, FCGX_Request *fcgx,
-				    int *init_error)
+int RGWREST::preprocess(struct req_state *s, RGWClientIO *cio)
+{
+  s->cio = cio;
+  s->script_uri = s->env->get("SCRIPT_URI", s->cct->_conf->rgw_script_uri.c_str());
+  s->request_uri = s->env->get("REQUEST_URI", s->cct->_conf->rgw_request_uri.c_str());
+  int pos = s->request_uri.find('?');
+  if (pos >= 0) {
+    s->request_params = s->request_uri.substr(pos + 1);
+    s->request_uri = s->request_uri.substr(0, pos);
+  }
+  s->host = s->env->get("HTTP_HOST");
+  if (g_conf->rgw_dns_name.length() && s->host) {
+    string h(s->host);
+
+    ldout(s->cct, 10) << "host=" << s->host << " rgw_dns_name=" << g_conf->rgw_dns_name << dendl;
+    pos = h.find(g_conf->rgw_dns_name);
+
+    if (g_conf->rgw_resolve_cname && pos < 0) {
+      string cname;
+      bool found;
+      int r = rgw_resolver->resolve_cname(h, cname, &found);
+      if (r < 0) {
+	dout(0) << "WARNING: rgw_resolver->resolve_cname() returned r=" << r << dendl;
+      }
+      if (found) {
+        dout(0) << "resolved host cname " << h << " -> " << cname << dendl;
+	h = cname;
+        pos = h.find(g_conf->rgw_dns_name);
+      }
+    }
+
+    if (pos > 0 && h[pos - 1] == '.') {
+      string encoded_bucket = "/";
+      encoded_bucket.append(h.substr(0, pos-1));
+      if (s->request_uri[0] != '/')
+	encoded_bucket.append("/'");
+      encoded_bucket.append(s->request_uri);
+      s->request_uri = encoded_bucket;
+    }
+  }
+
+  url_decode(s->request_uri, s->decoded_uri);
+  s->method = s->env->get("REQUEST_METHOD");
+  s->length = s->env->get("CONTENT_LENGTH");
+  if (s->length) {
+    if (*s->length == '\0')
+      s->content_length = 0;
+    else
+      s->content_length = atoll(s->length);
+  }
+
+  map<string, string>::iterator giter;
+  for (giter = generic_attrs_map.begin(); giter != generic_attrs_map.end(); ++giter) {
+    const char *env = s->env->get(giter->first.c_str());
+    if (env) {
+      s->generic_attrs[giter->second] = env;
+    }
+  }
+
+  s->http_auth = s->env->get("HTTP_AUTHORIZATION");
+
+  if (g_conf->rgw_print_continue) {
+    const char *expect = s->env->get("HTTP_EXPECT");
+    s->expect_cont = (expect && !strcasecmp(expect, "100-continue"));
+  }
+  s->op = op_from_method(s->method);
+
+  init_meta_info(s);
+
+  return 0;
+}
+
+RGWHandler *RGWREST::get_handler(RGWRados *store, struct req_state *s, RGWClientIO *cio,
+				 int *init_error)
 {
   RGWHandler *handler;
 
-  *init_error = RGWHandler_REST::preprocess(s, fcgx);
+  *init_error = preprocess(s, cio);
+  if (*init_error < 0)
+    return NULL;
 
-  if (s->prot_flags & RGW_REST_SWIFT)
-    handler = m_os_handler;
-  else if (s->prot_flags & RGW_REST_SWIFT_AUTH)
-    handler = m_os_auth_handler;
-  else
-    handler = m_s3_handler;
+  RGWRESTMgr *m = mgr.get_resource_mgr(s, s->decoded_uri);
+  if (!m) {
+    *init_error = -ERR_METHOD_NOT_ALLOWED;
+    return NULL;
+  }
 
-  handler->init(s, fcgx);
+  handler = m->get_handler(s);
+  if (!handler) {
+    *init_error = -ERR_METHOD_NOT_ALLOWED;
+    return NULL;
+  }
+  *init_error = handler->init(store, s, cio);
+  if (*init_error < 0)
+    return NULL;
 
   return handler;
 }
 
-void RGWHandler_REST::put_op(RGWOp *op)
-{
-  delete op;
-}
-
