@@ -53,6 +53,7 @@ static void usage()
   cout << "CLUSTER COMMANDS\n";
   cout << "  ceph health [detail]\n";
   cout << "  ceph quorum_status\n";
+  cout << "  ceph df [detail]\n";
   cout << "  ceph -m <mon-ip-or-host> mon_status\n";
   cout << "\n";
   cout << "AUTHENTICATION (AUTH) COMMANDS\n";
@@ -79,8 +80,8 @@ static void usage()
   cout << "  ceph osd down <osd-id>\n";
   cout << "  ceph osd in <osd-id>\n";
   cout << "  ceph osd out <osd-id>\n";
-  cout << "  ceph osd set <noout|noin|nodown|noup>\n";
-  cout << "  ceph osd unset <noout|noin|nodown|noup>\n";
+  cout << "  ceph osd set <noout|noin|nodown|noup|noscrub|nodeep-scrub>\n";
+  cout << "  ceph osd unset <noout|noin|nodown|noup|noscrub|nodeep-scrub>\n";
   cout << "  ceph osd pause\n";
   cout << "  ceph osd unpause\n";
   cout << "  ceph osd tell <osd-id or *> injectargs '--<switch> <value> [--<switch> <value>...]'\n";
@@ -336,8 +337,9 @@ int main(int argc, const char **argv)
 	  }
 
 	  bufferlist obl;
-	  if (do_command(ctx, cmd, indata, obl) < 0) {
-	    ret = 1;
+	  ret = do_command(ctx, cmd, indata, obl);
+	  if (ret < 0) {
+	    ret = -ret;
 	    break;
 	  }
 	  outbl.claim(obl);
